##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2017 Stanford University and the Authors
#
# Authors: Robert McGibbon
# Contributors: Matthew Harrigan
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################


from __future__ import print_function, division

import warnings
from itertools import combinations

import numpy as np
import pytest

from mdtraj.testing import eq
from mdtraj.utils import ensure_type
from mdtraj.utils import (import_, lengths_and_angles_to_box_vectors,
                          box_vectors_to_lengths_and_angles)
from mdtraj.utils.unit import in_units_of
from mdtraj.utils.validation import TypeCastPerformanceWarning

a = np.ones(10, dtype=np.float32)
b = np.ones((10, 10), dtype=np.float64, order='F')
random = np.random.RandomState(0)


def test_unitcell_0():
    result = lengths_and_angles_to_box_vectors(1, 1, 1, 90.0, 90.0, 90.0)
    expected = (np.array([1, 0, 0]), np.array([0., 1., 0.]), np.array([0., 0., 1.]))
    for (a, b) in zip(result, expected):
        np.testing.assert_array_almost_equal(a, b)


def test_unitcell_1():
    # try round-tripping some random lengths and angles through
    # lengths_and_angles_to_box_vectors and box_vectors_to_lengths_and_angles,
    # and make sure we get back to where we started
    for _ in range(10):
        arg = np.hstack((random.rand(3), random.uniform(70, 110, size=3)))
        vectors = lengths_and_angles_to_box_vectors(*arg)
        out = box_vectors_to_lengths_and_angles(*vectors)
        np.testing.assert_array_almost_equal(arg, out)


def test_ensure_type_1():
    ensure_type(a, np.float32, 1, '', length=10)


def test_ensure_type_2():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")

        val = ensure_type(a, np.float64, 1, '', length=10)

        assert val.dtype == np.float64
        assert a.dtype == np.float32  # a should not be changed
        assert len(w) == 1
        assert issubclass(w[-1].category, TypeCastPerformanceWarning)


def test_ensure_type_25():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")

        val = ensure_type(a, np.float64, 1, '', length=10, warn_on_cast=False)

        assert val.dtype == np.float64
        assert a.dtype == np.float32  # a should not be changed
        assert len(w) == 0  # no warning since we set warn_on_cast to False


def test_ensure_type_3():
    with pytest.raises(ValueError):
        ensure_type(a, np.float32, 1, '', length=11)


def test_ensure_type_4():
    ensure_type(None, np.float64, 1, '', length=11, can_be_none=True)


def test_ensure_type_5():
    with pytest.raises(ValueError):
        ensure_type(a, np.float32, 1, '', length=11, can_be_none=True)


def test_ensure_type_6():
    val = ensure_type(b, np.float64, 2, '', shape=(10, 10))
    assert val.flags.c_contiguous is True


def test_ensure_type_7():
    c = ensure_type(a, np.float32, ndim=2, name='', add_newaxis_on_deficient_ndim=True)
    assert c.shape == (1, len(a))


def test_ensure_type_8():
    c = ensure_type(np.zeros((5, 10)), np.float32, ndim=2, name='', shape=(None, 10))
    assert c.shape == (5, 10)


def test_ensure_type_9():
    with pytest.raises(ValueError):
        c = ensure_type(np.zeros((5, 11)), np.float32, ndim=2, name='', shape=(None, 10))


def test_ensure_type_10():
    with pytest.raises(ValueError):
        c = ensure_type([0, 1], np.float32, ndim=2, name='')


def test_ensure_type_11():
    c = ensure_type(0, np.float32, ndim=1, name='', add_newaxis_on_deficient_ndim=True)
    assert c.shape == (1,)


def test_ensure_type_12():
    with pytest.raises(TypeError):
        ensure_type(np.zeros((2, 2)), np.float32, ndim=3)


def test_ensure_type_13():
    with pytest.raises(ValueError):
        ensure_type(np.zeros((2, 2)), np.float32, ndim=2, name='', shape=(None, None, None))


def test_ensure_type_14():
    # test that the generators work
    value = ensure_type(combinations(range(10), 2), int, ndim=2, name='')
    assert isinstance(value, np.ndarray)
    ref = np.array(list(combinations(range(10), 2)))
    eq(value, ref)


def test_ensure_type_15():
    # test that lists
    x = [1, 2, 3]
    value = ensure_type(x, int, ndim=1, name='')
    ref = np.array(x)
    eq(value, ref)


def test_delay_import_fail_1():
    with pytest.raises(ImportError):
        import_('sdfsdfsfsfdsdf')


def test_delay_import():
    import_('scipy.sparse')


def test_unit_0():
    a = np.array([1.0])
    b = in_units_of(a, 'nanometers', 'angstroms', inplace=False)
    c = in_units_of(a, 'angstroms', 'nanometers', inplace=False)
    eq(b, np.array([10.0]))
    eq(c, np.array([0.1]))
    assert a.ctypes.data != b.ctypes.data
    assert a.ctypes.data != c.ctypes.data


def test_unit_1():
    a = np.array([1.0])
    b = in_units_of(a, 'nanometers', 'angstroms', inplace=True)
    eq(a, np.array([10.0]))
    eq(b, np.array([10.0]))
    # a and b point to the same memory
    assert a.ctypes.data == b.ctypes.data


def test_unit_2():
    a = np.array([1.0])
    a.flags['WRITEABLE'] = False
    b = in_units_of(a, 'nanometers', 'angstroms', inplace=True)

    eq(b, np.array([10.0]))
    # a and b do not point to the same memory, since a isn't writeable
    assert a.ctypes.data != b.ctypes.data


def test_unit_3():
    eq(1000000.0, in_units_of(1, 'meter**2/second', 'nanometers**2/picosecond'))



from __future__ import print_function

import mdtraj as md
import numpy as np
import scipy.special
from mdtraj.geometry import compute_drid
from scipy.spatial.distance import euclidean, pdist, squareform

# To keep backwards compatibility
try:
    from scipy.stats import nanmean
except ImportError:
    from numpy import nanmean


def test_drid_1():
    n_frames = 1
    n_atoms = 20
    top = md.Topology()
    chain = top.add_chain()
    residue = top.add_residue('X', chain)
    for i in range(n_atoms):
        top.add_atom('X', None, residue)

    t = md.Trajectory(xyz=np.random.RandomState(0).randn(n_frames, n_atoms, 3),
                      topology=top)
    # t contains no bonds
    got = compute_drid(t).reshape(n_frames, n_atoms, 3)

    for i in range(n_atoms):
        others = set(range(n_atoms)) - set([i])
        rd = 1 / np.array([euclidean(t.xyz[0, i], t.xyz[0, e]) for e in others])

        mean = np.mean(rd)
        second = np.mean((rd - mean) ** 2) ** (0.5)
        third = scipy.special.cbrt(np.mean((rd - mean) ** 3))

        ref = np.array([mean, second, third])
        np.testing.assert_array_almost_equal(got[0, i], ref, decimal=5)


def test_drid_2():
    n_frames = 3
    n_atoms = 11
    n_bonds = 5
    top = md.Topology()
    chain = top.add_chain()
    residue = top.add_residue('X', chain)
    for i in range(n_atoms):
        top.add_atom('X', None, residue)

    random = np.random.RandomState(0)
    bonds = random.randint(n_atoms, size=(n_bonds, 2))
    for a, b in bonds:
        top.add_bond(top.atom(a), top.atom(b))

    t = md.Trajectory(xyz=random.randn(n_frames, n_atoms, 3), topology=top)
    got = compute_drid(t).reshape(n_frames, n_atoms, 3)

    for i in range(n_frames):
        recip = 1 / squareform(pdist(t.xyz[i]))
        recip[np.diag_indices(n=recip.shape[0])] = np.nan
        recip[bonds[:, 0], bonds[:, 1]] = np.nan
        recip[bonds[:, 1], bonds[:, 0]] = np.nan

        mean = nanmean(recip, axis=0)
        second = nanmean((recip - mean) ** 2, axis=0) ** (0.5)
        third = scipy.special.cbrt(nanmean((recip - mean) ** 3, axis=0))

        np.testing.assert_array_almost_equal(got[i, :, 0], mean, decimal=5)
        np.testing.assert_array_almost_equal(got[i, :, 1], second, decimal=5)
        np.testing.assert_array_almost_equal(got[i, :, 2], third, decimal=5)



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2017 Stanford University and the Authors
#
# Authors: Christian Schwantes
# Contributors: Robert McGibbon
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################

from __future__ import print_function

import itertools

import mdtraj as md
import numpy as np
from mdtraj.testing import eq


def test_contact_0(get_fn):
    pdb = md.load(get_fn('bpti.pdb'))
    contacts = np.loadtxt(get_fn('contacts.dat')).astype(int)

    ca, ca_pairs = md.compute_contacts(pdb, contacts, scheme='ca')
    closest, closest_pairs = md.compute_contacts(pdb, contacts, scheme='closest')
    closest_heavy, closest_heavy_pairs = md.compute_contacts(pdb, contacts, scheme='closest-heavy')
    sidechain, sidechain_pairs = md.compute_contacts(pdb, contacts, scheme='sidechain')
    sidechain_heavy, sidechain_heavy_pairs = md.compute_contacts(pdb, contacts, scheme='sidechain-heavy')

    ref_ca = np.loadtxt(get_fn('cc_ca.dat'))
    ref_closest = np.loadtxt(get_fn('cc_closest.dat'))
    ref_closest_heavy = np.loadtxt(get_fn('cc_closest-heavy.dat'))
    ref_sidechain = np.loadtxt(get_fn('cc_sidechain.dat'))
    ref_sidechain_heavy = np.loadtxt(get_fn('cc_sidechain-heavy.dat'))

    eq(ref_ca, ca.flatten())
    eq(ref_closest, closest.flatten())
    eq(ref_closest_heavy, closest_heavy.flatten())
    eq(ref_sidechain, sidechain.flatten())
    eq(ref_sidechain_heavy, sidechain_heavy.flatten())
    eq(contacts, ca_pairs)
    eq(contacts, closest_pairs)
    eq(contacts, closest_heavy_pairs)
    eq(contacts, sidechain_pairs)
    eq(contacts, sidechain_heavy_pairs)


def test_contact_1(get_fn):
    pdb = md.load(get_fn('bpti.pdb'))
    dists, pairs = md.compute_contacts(pdb)
    for r0, r1 in pairs:
        # are these valid residue indices?
        pdb.topology.residue(r0)
        pdb.topology.residue(r1)

        assert not (abs(r0 - r1) < 3)

    maps = md.geometry.squareform(dists, pairs)
    for i, (r0, r1) in enumerate(pairs):
        for t in range(pdb.n_frames):
            eq(maps[t, r0, r1], dists[t, i])


def test_contact_2(get_fn):
    pdb = md.load(get_fn('1vii_sustiva_water.pdb'))
    dists, pairs = md.compute_contacts(pdb, scheme='closest')
    for r0, r1 in pairs:
        assert pdb.topology.residue(r0).name != 'HOH'
        assert pdb.topology.residue(r1).name != 'HOH'

    # spot check one of the pairs
    r0, r1 = pairs[10]
    atoms_r0 = [a.index for a in pdb.topology.residue(r0).atoms]
    atoms_r1 = [a.index for a in pdb.topology.residue(r1).atoms]

    atomdist = md.compute_distances(pdb, list(itertools.product(atoms_r0, atoms_r1)))

    np.testing.assert_array_equal(dists[:, 10], np.min(atomdist, axis=1))

    maps = md.geometry.squareform(dists, pairs)
    for i, (r0, r1) in enumerate(pairs):
        for t in range(pdb.n_frames):
            eq(maps[t, r0, r1], dists[t, i])


def test_contact_3(get_fn):
    pdb = md.load(get_fn('bpti.pdb'))
    beta = 20
    dists, pairs = md.compute_contacts(pdb, soft_min=True, soft_min_beta=beta)

    maps = md.geometry.squareform(dists, pairs)
    for i, (r0, r1) in enumerate(pairs):
        for t in range(pdb.n_frames):
            assert np.allclose(beta / np.log(np.sum(np.exp(beta / maps[t, r0, r1]))), dists[t, i])
            
def test_contact_4(get_fn):
    pdb = md.load(get_fn('1am7_protein.pdb'))   # protonated and including at least one glycine residue (which has no heavy atoms in its sidechain)
    contacts = md.compute_contacts(pdb, contacts='all', scheme='sidechain-heavy')   # test passes if this doesn't raise an exception



import re
import mdtraj
from mdtraj.testing.docstrings import docstring_verifiers, import_all_modules
from mdtraj.testing.docstrings import import_all_modules
import pytest

SKIP_MODULES = [
    r'mdtraj\.utils\.external',
    r'mdtraj\.utils\.six',
    r'mdtraj\.utils\.unit\.unit_math',
    r'mdtraj\.utils\.unit\.baseunit',
    r'mdtraj\.utils\.unit\.prefix',
    r'mdtraj\.utils\.unit\.unit',
    r'mdtraj\.utils\.unit\.quantity',
    r'mdtraj\.utils\.unit\.mymatrix',
    r'mdtraj\.formats\.lh5',
    r'mdtraj\.formats\.hdf5',
    r'mdtraj\.formats\.pdb\.pdbstructure',
    r'mdtraj\.scripts',
    r'mdtraj\.testing\.docscrape',
    r'mdtraj\.io',
]

MODULES = import_all_modules(mdtraj)


@pytest.fixture(params=MODULES, ids=lambda x: x.__name__)
def module(request):
    mod = request.param
    if any(re.search(s, mod.__name__) for s in SKIP_MODULES):
        pytest.skip("Skip checking certain modules' docstrings")

    return mod


def test_docstrings(module):
    for test in docstring_verifiers(module):
        test()



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2013 Stanford University and the Authors
#
# Authors: Robert McGibbon
# Contributors:
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################


"""
Code to test the mdconvert script. These tests take about two minutes to run.

This checks all pairs for formats, converting from format x -> format y. it
also tries using striding to subsample the trajectory and atom_indices, so it
does significant integration testing of the XXXTrajectoryFile modules as well.
"""

import os
import sys
import numpy as np

import mdtraj as md
from mdtraj import element
from mdtraj.testing import eq
import pytest
import subprocess

on_win = (sys.platform == 'win32')
on_py3 = (sys.version_info >= (3, 0))


@pytest.fixture()
def traj(tmpdir):
    xyz = np.around(np.random.randn(10, 5, 3).astype(np.float32), 2)
    topology = md.Topology()
    chain = topology.add_chain()
    residue = topology.add_residue('ALA', chain)
    topology.add_atom('CA', element.carbon, residue)
    topology.add_atom('HG1', element.hydrogen, residue)
    topology.add_atom('SG', element.sulfur, residue)
    topology.add_atom('OD1', element.oxygen, residue)
    topology.add_atom('NE', element.nitrogen, residue)

    time = np.arange(10) ** 2
    unitcell_lengths = np.array([[1.1, 1.2, 1.3]] * 10)
    unitcell_angles = np.array([[90, 90, 95]] * 10)

    traj = md.Trajectory(xyz, topology=topology, time=time,
                         unitcell_lengths=unitcell_lengths,
                         unitcell_angles=unitcell_angles)

    fn = '{}/ref.h5'.format(tmpdir)
    traj.save(fn)
    return traj, fn, str(tmpdir)


def test_index(traj):
    # Check that extracting a specific index works
    traj, in_fn, tmpdir = traj
    out_fn = '{}/frame4.pdb'.format(tmpdir)
    subprocess.check_call(['mdconvert', in_fn, '-i', '4', '-o', out_fn])
    frame4 = md.load(out_fn)
    eq(frame4.xyz, traj[4].xyz)


def test_slice(traj):
    # Check that extracting a specific slice works
    traj, in_fn, tmpdir = traj
    out_fn = '{}/frame13.pdb'.format(tmpdir)
    subprocess.check_call(['mdconvert', in_fn, '-i', '1:5:2', '-o', out_fn])
    frame13 = md.load(out_fn)
    eq(frame13.xyz, traj[1:5:2].xyz)


extensions = [
    'xtc', 'dcd', 'binpos', 'trr', 'nc',
    'pdb', 'h5', 'lh5', 'netcdf'
]


@pytest.fixture(params=extensions, ids=lambda x: 'from-' + x)
def extension(request):
    if on_win and on_py3 and request.param == 'lh5':
        pytest.skip('No lh5 on windows py3')
    return request.param


def test_pairwise(traj, extension):
    """ensure that the xyz coordinates are preserved by a trip
       from python -> save in format X -> mdconvert to format Y -> python
    """
    traj, _, tmpdir = traj
    ext1 = extension

    # save one copy of traj for use as a topology file
    topology_fn = "{}/topology.pdb".format(tmpdir)
    traj[0].save(topology_fn)

    # save a .dat file for the atom_indices so that we can test
    # mdconvert's atom_indices flag
    atom_indices = np.array([0, 3])
    atom_indices_fn = "{}/atom_indices.dat".format(tmpdir)
    np.savetxt(atom_indices_fn, atom_indices, fmt='%d')

    in_fn = "{}/traj.{}".format(tmpdir, ext1)
    traj.save(in_fn)
    working_dir = '{}/from-{}'.format(tmpdir, ext1)
    os.mkdir(working_dir)

    for ext2 in extensions:
        print(ext2)
        out_fn = 'traj.{}'.format(ext2)

        command1 = ['mdconvert', in_fn, '-o', out_fn, '-c 6']
        if ext2 in ['pdb', 'h5', 'lh5']:
            # if we're saving a pdb or h5, we need to give it a topology too
            command1 += ['-t', topology_fn]

        # TODO: test fixture
        subprocess.check_call(command1, cwd=working_dir)

        # Use the --atom_indices flag to mdconvert
        command2 = command1 + ['-a', atom_indices_fn]
        command2[3] = 'subset.' + out_fn  # make sure the output goes to a different file
        subprocess.check_call(command2, cwd=working_dir)

        # Use the --stride 3 flag
        command3 = command1 + ['-s 3']
        command3[3] = 'stride.' + out_fn  # change the out filename, so they don't clobbed
        subprocess.check_call(command3, cwd=working_dir)

        # ensure that the xyz coordinates are preserved by a trip
        # from python -> save in format X -> mdconvert to format Y -> python
        load_kwargs_check1 = {}
        load_kwargs_check2 = {}
        if ext2 not in ['pdb', 'h5', 'lh5']:
            load_kwargs_check1['top'] = traj.topology
            load_kwargs_check2['top'] = traj.topology.subset(atom_indices)

        out1 = md.load(os.path.join(working_dir, out_fn), **load_kwargs_check1)
        out2 = md.load(os.path.join(working_dir, 'subset.' + out_fn), **load_kwargs_check2)
        out3 = md.load(os.path.join(working_dir, 'stride.' + out_fn), **load_kwargs_check1)

        if ext1 in ['lh5'] or ext2 in ['lh5']:
            decimal = 3
        else:
            decimal = 6
        eq(out1.xyz, traj.xyz, decimal=decimal)
        eq(out2.xyz, traj.xyz[:, atom_indices], decimal=decimal)
        eq(out3.xyz, traj.xyz[::3], decimal=decimal)

        if ext1 not in ['binpos', 'lh5'] and ext2 not in ['binpos', 'lh5']:
            # binpos doesn't save unitcell information
            eq(out1.unitcell_vectors, traj.unitcell_vectors, decimal=2)
            eq(out2.unitcell_vectors, traj.unitcell_vectors, decimal=2)
            eq(out3.unitcell_vectors, traj.unitcell_vectors[::3], decimal=2)

        if all(e in ['xtc', 'trr', 'nc', 'h5'] for e in [ext1, ext2]):
            # these formats contain time information
            eq(out1.time, traj.time)
            eq(out2.time, traj.time)
            eq(out3.time, traj.time[::3])

        if ext2 in ['pdb', 'h5', 'lh5']:
            # these formats contain a topology in the file that was
            # read from disk
            eq(out1.topology, traj.topology)
            eq(out2.topology, traj.topology.subset(atom_indices))
            eq(out3.topology, traj.topology)


def test_mdconvert_alanine(tmpdir, get_fn):
    command = ['mdconvert', get_fn('alanine-dipeptide-explicit.binpos'),
               '--top', get_fn('alanine-dipeptide-explicit.prmtop'),
               '-o', 'out.dcd']
    subprocess.check_call(command, cwd=str(tmpdir))
    t = md.load('{}/out.dcd'.format(tmpdir), top=get_fn('alanine-dipeptide-explicit.prmtop'))
    t2 = md.load(get_fn('alanine-dipeptide-explicit.binpos'), top=get_fn('alanine-dipeptide-explicit.prmtop'))

    eq(t.xyz, t2.xyz)
    eq(t.topology, t2.topology)



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2013 Stanford University and the Authors
#
# Authors: Jason Swails
# Contributors:
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################


import os
import tempfile

import mdtraj as md
import numpy as np
import pytest
from mdtraj.formats import AmberRestartFile, AmberNetCDFRestartFile
from mdtraj.testing import eq

fd1, temp1 = tempfile.mkstemp(suffix='.rst7')
fd2, temp2 = tempfile.mkstemp(suffix='.ncrst')
os.close(fd1)
os.close(fd2)


def teardown_module(module):
    """remove the temporary file created by tests in this file
    this gets automatically called by pytest"""
    if os.path.exists(temp1): os.unlink(temp1)
    if os.path.exists(temp2): os.unlink(temp2)


def test_read_after_close(get_fn):
    f = AmberNetCDFRestartFile(get_fn('ncinpcrd.rst7'))
    assert eq(f.n_atoms, 2101)
    assert eq(f.n_frames, 1)

    f.close()

    with pytest.raises(IOError):
        f.read()


def test_shape(get_fn):
    with AmberRestartFile(get_fn('inpcrd')) as f:
        xyz, time, lengths, angles = f.read()

    assert eq(xyz.shape, (1, 2101, 3))
    assert eq(time.shape, (1,))
    assert eq(lengths, np.asarray([[30.2642725] * 3]))
    assert eq(angles, np.asarray([[109.471219] * 3]))


def test_shape_2(get_fn):
    with AmberNetCDFRestartFile(get_fn('ncinpcrd.rst7')) as f:
        xyz, time, lengths, angles = f.read()

    assert eq(xyz.shape, (1, 2101, 3))
    assert eq(time.shape, (1,))
    assert eq(lengths, np.asarray([[30.2642725] * 3]))
    assert eq(angles, np.asarray([[109.471219] * 3]))


def test_read_write_1():
    xyz = np.random.randn(1, 10, 3)
    time = np.random.randn(1)
    boxlengths = np.random.randn(1, 3)
    boxangles = np.random.randn(1, 3)

    with AmberRestartFile(temp1, 'w', force_overwrite=True) as f:
        f.write(xyz, time, boxlengths, boxangles)

    with AmberRestartFile(temp1) as f:
        a, b, c, d = f.read()
        assert eq(a, xyz)
        assert eq(b, time)
        assert eq(c, boxlengths)
        assert eq(d, boxangles)


def test_read_write_2():
    xyz = np.random.randn(1, 10, 3)
    time = np.random.randn(1)
    boxlengths = np.random.randn(1, 3)
    boxangles = np.random.randn(1, 3)

    with AmberNetCDFRestartFile(temp2, 'w', force_overwrite=True) as f:
        f.write(xyz, time, boxlengths, boxangles)

    with AmberNetCDFRestartFile(temp2) as f:
        a, b, c, d = f.read()
        assert eq(a, xyz)
        assert eq(b, time)
        assert eq(c, boxlengths)
        assert eq(d, boxangles)


def test_read_write_3(get_fn):
    traj = md.load(get_fn('frame0.nc'), top=get_fn('native.pdb'))
    traj[0].save(temp1)
    assert os.path.exists(temp1)
    rsttraj = md.load(temp1, top=get_fn('native.pdb'))
    eq(rsttraj.xyz, traj[0].xyz)
    os.unlink(temp1)
    traj.save(temp1)
    for i in range(traj.n_frames):
        assert os.path.exists('%s.%03d' % (temp1, i + 1))
        os.unlink('%s.%03d' % (temp1, i + 1))


def test_read_write_4(get_fn):
    traj = md.load(get_fn('frame0.nc'), top=get_fn('native.pdb'))
    traj[0].save(temp2)
    assert os.path.exists(temp2)
    rsttraj = md.load(temp2, top=get_fn('native.pdb'))
    eq(rsttraj.xyz, traj[0].xyz)
    os.unlink(temp2)
    traj.save(temp2)
    for i in range(traj.n_frames):
        assert os.path.exists('%s.%03d' % (temp2, i + 1))
        os.unlink('%s.%03d' % (temp2, i + 1))



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2017 Stanford University and the Authors
#
# Authors: Robert McGibbon
# Contributors:
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################

import os
import shutil

import numpy as np
import pytest

import mdtraj as md
from mdtraj.formats import HDF5TrajectoryFile, NetCDFTrajectoryFile
from mdtraj.reporters import HDF5Reporter, NetCDFReporter, DCDReporter, XTCReporter
from mdtraj.testing import eq

try:
    from simtk.unit import nanometers, kelvin, picoseconds, femtoseconds
    from simtk.openmm import LangevinIntegrator, Platform
    from simtk.openmm.app import PDBFile, ForceField, Simulation, CutoffNonPeriodic, CutoffPeriodic, HBonds, CheckpointReporter

    HAVE_OPENMM = True
except ImportError:
    HAVE_OPENMM = False

# special pytest global to mark all tests in this module
pytestmark = pytest.mark.skipif(not HAVE_OPENMM, reason='test_reporter.py needs OpenMM.')


def test_reporter(tmpdir, get_fn):
    pdb = PDBFile(get_fn('native.pdb'))
    forcefield = ForceField('amber99sbildn.xml', 'amber99_obc.xml')
    # NO PERIODIC BOUNDARY CONDITIONS
    system = forcefield.createSystem(pdb.topology, nonbondedMethod=CutoffNonPeriodic,
                                     nonbondedCutoff=1.0 * nanometers, constraints=HBonds, rigidWater=True)
    integrator = LangevinIntegrator(300 * kelvin, 1.0 / picoseconds, 2.0 * femtoseconds)
    integrator.setConstraintTolerance(0.00001)

    platform = Platform.getPlatformByName('Reference')
    simulation = Simulation(pdb.topology, system, integrator, platform)
    simulation.context.setPositions(pdb.positions)

    simulation.context.setVelocitiesToTemperature(300 * kelvin)

    tmpdir = str(tmpdir)
    hdf5file = os.path.join(tmpdir, 'traj.h5')
    ncfile = os.path.join(tmpdir, 'traj.nc')
    dcdfile = os.path.join(tmpdir, 'traj.dcd')
    xtcfile = os.path.join(tmpdir, 'traj.xtc')

    reporter = HDF5Reporter(hdf5file, 2, coordinates=True, time=True,
                            cell=True, potentialEnergy=True, kineticEnergy=True, temperature=True,
                            velocities=True)
    reporter2 = NetCDFReporter(ncfile, 2, coordinates=True, time=True, cell=True)
    reporter3 = DCDReporter(dcdfile, 2)
    reporter4 = XTCReporter(xtcfile, 2)

    simulation.reporters.append(reporter)
    simulation.reporters.append(reporter2)
    simulation.reporters.append(reporter3)
    simulation.reporters.append(reporter4)
    simulation.step(100)

    reporter.close()
    reporter2.close()
    reporter3.close()
    reporter4.close()

    with HDF5TrajectoryFile(hdf5file) as f:
        got = f.read()
        eq(got.temperature.shape, (50,))
        eq(got.potentialEnergy.shape, (50,))
        eq(got.kineticEnergy.shape, (50,))
        eq(got.coordinates.shape, (50, 22, 3))
        eq(got.velocities.shape, (50, 22, 3))
        eq(got.cell_lengths.shape, (50, 3))
        eq(got.cell_angles.shape, (50, 3))
        eq(got.time, 0.002 * 2 * (1 + np.arange(50)))
        assert f.topology == md.load(get_fn('native.pdb')).top

    with NetCDFTrajectoryFile(ncfile) as f:
        xyz, time, cell_lengths, cell_angles = f.read()
        eq(cell_lengths.shape, (50, 3))
        eq(cell_angles.shape, (50, 3))
        eq(time, 0.002 * 2 * (1 + np.arange(50)))

    hdf5_traj = md.load(hdf5file)
    dcd_traj = md.load(dcdfile, top=get_fn('native.pdb'))
    netcdf_traj = md.load(ncfile, top=get_fn('native.pdb'))
    xtc_traj = md.load(xtcfile, top=get_fn('native.pdb'))

    # we don't have to convert units here, because md.load already
    # handles that
    eq(hdf5_traj.xyz, netcdf_traj.xyz)
    eq(hdf5_traj.unitcell_vectors, netcdf_traj.unitcell_vectors)
    eq(hdf5_traj.time, netcdf_traj.time)
    eq(xtc_traj.time, netcdf_traj.time)

    eq(dcd_traj.xyz, hdf5_traj.xyz)
    eq(xtc_traj.xyz, dcd_traj.xyz, decimal=3)
    eq(dcd_traj.unitcell_vectors, hdf5_traj.unitcell_vectors)


def test_reporter_subset(tmpdir, get_fn):
    pdb = PDBFile(get_fn('native2.pdb'))
    pdb.topology.setUnitCellDimensions([2, 2, 2])
    forcefield = ForceField('amber99sbildn.xml', 'amber99_obc.xml')
    system = forcefield.createSystem(pdb.topology, nonbondedMethod=CutoffPeriodic,
                                     nonbondedCutoff=1 * nanometers, constraints=HBonds, rigidWater=True)
    integrator = LangevinIntegrator(300 * kelvin, 1.0 / picoseconds, 2.0 * femtoseconds)
    integrator.setConstraintTolerance(0.00001)

    platform = Platform.getPlatformByName('Reference')
    simulation = Simulation(pdb.topology, system, integrator, platform)
    simulation.context.setPositions(pdb.positions)

    simulation.context.setVelocitiesToTemperature(300 * kelvin)

    tmpdir = str(tmpdir)
    hdf5file = os.path.join(tmpdir, 'traj.h5')
    ncfile = os.path.join(tmpdir, 'traj.nc')
    dcdfile = os.path.join(tmpdir, 'traj.dcd')
    xtcfile = os.path.join(tmpdir, 'traj.xtc')

    atomSubset = [0, 1, 2, 4, 5]

    reporter = HDF5Reporter(hdf5file, 2, coordinates=True, time=True,
                            cell=True, potentialEnergy=True, kineticEnergy=True, temperature=True,
                            velocities=True, atomSubset=atomSubset)
    reporter2 = NetCDFReporter(ncfile, 2, coordinates=True, time=True,
                               cell=True, atomSubset=atomSubset)
    reporter3 = DCDReporter(dcdfile, 2, atomSubset=atomSubset)
    reporter4 = XTCReporter(xtcfile, 2, atomSubset=atomSubset)

    simulation.reporters.append(reporter)
    simulation.reporters.append(reporter2)
    simulation.reporters.append(reporter3)
    simulation.reporters.append(reporter4)
    simulation.step(100)

    reporter.close()
    reporter2.close()
    reporter3.close()
    reporter4.close()

    t = md.load(get_fn('native.pdb'))
    t.restrict_atoms(atomSubset)

    with HDF5TrajectoryFile(hdf5file) as f:
        got = f.read()
        eq(got.temperature.shape, (50,))
        eq(got.potentialEnergy.shape, (50,))
        eq(got.kineticEnergy.shape, (50,))
        eq(got.coordinates.shape, (50, len(atomSubset), 3))
        eq(got.velocities.shape, (50, len(atomSubset), 3))
        eq(got.cell_lengths, 2 * np.ones((50, 3)))
        eq(got.cell_angles, 90 * np.ones((50, 3)))
        eq(got.time, 0.002 * 2 * (1 + np.arange(50)))
        assert f.topology == md.load(get_fn('native.pdb'), atom_indices=atomSubset).topology

    with NetCDFTrajectoryFile(ncfile) as f:
        xyz, time, cell_lengths, cell_angles = f.read()
        eq(cell_lengths, 20 * np.ones((50, 3)))
        eq(cell_angles, 90 * np.ones((50, 3)))
        eq(time, 0.002 * 2 * (1 + np.arange(50)))
        eq(xyz.shape, (50, len(atomSubset), 3))

    hdf5_traj = md.load(hdf5file)
    dcd_traj = md.load(dcdfile, top=hdf5_traj)
    netcdf_traj = md.load(ncfile, top=hdf5_traj)
    xtc_traj = md.load(xtcfile, top=hdf5_traj)

    # we don't have to convert units here, because md.load already handles that
    eq(hdf5_traj.xyz, netcdf_traj.xyz)
    eq(hdf5_traj.unitcell_vectors, netcdf_traj.unitcell_vectors)
    eq(hdf5_traj.time, netcdf_traj.time)
    eq(xtc_traj.time, netcdf_traj.time)

    eq(dcd_traj.xyz, hdf5_traj.xyz)
    eq(xtc_traj.xyz, hdf5_traj.xyz)
    eq(dcd_traj.unitcell_vectors, hdf5_traj.unitcell_vectors)


def test_xtc_reporter_append(tmpdir, get_fn):
    pdb = PDBFile(get_fn('native.pdb'))
    forcefield = ForceField('amber99sbildn.xml', 'amber99_obc.xml')
    # NO PERIODIC BOUNDARY CONDITIONS
    system = forcefield.createSystem(pdb.topology, nonbondedMethod=CutoffNonPeriodic,
                                     nonbondedCutoff=1.0 * nanometers, constraints=HBonds, rigidWater=True)
    integrator = LangevinIntegrator(300 * kelvin, 1.0 / picoseconds, 2.0 * femtoseconds)
    integrator.setConstraintTolerance(0.00001)

    platform = Platform.getPlatformByName('Reference')
    simulation = Simulation(pdb.topology, system, integrator, platform)
    simulation.context.setPositions(pdb.positions)

    simulation.context.setVelocitiesToTemperature(300 * kelvin)

    tmpdir = str(tmpdir)
    xtcfile = os.path.join(tmpdir, 'traj.xtc')
    xtcfile_cp = os.path.join(tmpdir, 'traj_cp.xtc')
    checkpoint = os.path.join(tmpdir, 'checkpoint.chk')
    reporter = XTCReporter(xtcfile, 2)
    simulation.reporters.append(reporter)
    simulation.reporters.append(CheckpointReporter(checkpoint, 10))
    simulation.step(10)
    reporter.close()
    shutil.copyfile(xtcfile, xtcfile_cp)
    system = forcefield.createSystem(pdb.topology, nonbondedMethod=CutoffNonPeriodic,
                                     nonbondedCutoff=1.0 * nanometers, constraints=HBonds, rigidWater=True)
    integrator = LangevinIntegrator(300 * kelvin, 1.0 / picoseconds, 2.0 * femtoseconds)
    integrator.setConstraintTolerance(0.00001)

    platform = Platform.getPlatformByName('Reference')
    simulation = Simulation(pdb.topology, system, integrator, platform)
    simulation.loadCheckpoint(checkpoint)
    reporter = XTCReporter(xtcfile, 2, append=True)
    simulation.reporters.append(reporter)
    simulation.step(10)
    reporter.close()
    xtc_traj = md.load(xtcfile, top=get_fn('native.pdb'))
    xtc_traj_cp = md.load(xtcfile_cp, top=get_fn('native.pdb'))
    eq(xtc_traj.xyz[:5], xtc_traj_cp.xyz)
    eq(xtc_traj.n_frames, 10)
    eq(xtc_traj_cp.n_frames, 5)
    eq(xtc_traj.time[:5], xtc_traj_cp.time)



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2013 Stanford University and the Authors
#
# Authors: Robert McGibbon
# Contributors:
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################

import numpy as np

from mdtraj import io
from mdtraj.formats import DCDTrajectoryFile, BINPOSTrajectoryFile
from mdtraj.testing import eq

# frame0.binpos was generated from frame0.dcd using
# VMD. The binpos file has one more frame than the dcd file
# because VMD also saved the PDB coordinates
#fn_binpos = get_fn('frame0.binpos')
#fn_dcd = get_fn('frame0.dcd')


def test_read_0(get_fn):
    fn_binpos = get_fn('frame0.binpos')
    fn_dcd = get_fn('frame0.dcd')
    with BINPOSTrajectoryFile(fn_binpos) as f:
        xyz = f.read()
    with DCDTrajectoryFile(fn_dcd) as f:
        xyz2 = f.read()[0]
    xyz3 = io.loadh(get_fn('frame0.binpos.h5'), 'xyz')

    assert eq(xyz[1:], xyz2)
    assert eq(xyz, xyz3)


def test_read_stride(get_fn):
    # Read a binpos with stride=3
    fn_binpos = get_fn('frame0.binpos')
    with BINPOSTrajectoryFile(fn_binpos) as f:
        xyz = f.read(stride=3)
    xyz2 = io.loadh(get_fn('frame0.binpos.h5'), 'xyz')
    assert eq(xyz, xyz2[::3])


def test_read_stride_2(get_fn):
    # Read a binpos with stride=3 when n_frames is supplied (different code path)
    fn_binpos = get_fn('frame0.binpos')
    with BINPOSTrajectoryFile(fn_binpos) as f:
        xyz = f.read(n_frames=1000, stride=3)
    xyz2 = io.loadh(get_fn('frame0.binpos.h5'), 'xyz')
    assert eq(xyz, xyz2[::3])


def test_read_atom_indices(get_fn):
    "Read a binpos with atom_indices as a list"
    fn_binpos = get_fn('frame0.binpos')
    with BINPOSTrajectoryFile(fn_binpos) as f:
        xyz = f.read(atom_indices=[0, 1, 2])
    xyz2 = io.loadh(get_fn('frame0.binpos.h5'), 'xyz')
    assert eq(xyz, xyz2[:, [0, 1, 2], :])


def test_read_atom_indices_slice(get_fn):
    "Read a binpos with atom_indices as a slice"
    fn_binpos = get_fn('frame0.binpos')
    with BINPOSTrajectoryFile(fn_binpos) as f:
        xyz = f.read(atom_indices=slice(0, 10, None))
    xyz2 = io.loadh(get_fn('frame0.binpos.h5'), 'xyz')
    assert eq(xyz, xyz2[:, 0:10, :])


def test_read_1(get_fn):
    fn_binpos = get_fn('frame0.binpos')
    fn_dcd = get_fn('frame0.dcd')
    with BINPOSTrajectoryFile(fn_binpos, chunk_size_multiplier=0.5) as f:
        xyz = f.read()
    with DCDTrajectoryFile(fn_dcd) as f:
        xyz2 = f.read()[0]
    xyz3 = io.loadh(get_fn('frame0.binpos.h5'), 'xyz')

    assert eq(xyz[1:], xyz2)
    assert eq(xyz, xyz3)


def test_read_2(get_fn):
    fn_binpos = get_fn('frame0.binpos')
    fn_dcd = get_fn('frame0.dcd')
    with BINPOSTrajectoryFile(fn_binpos, chunk_size_multiplier=10) as f:
        xyz = f.read()
    with DCDTrajectoryFile(fn_dcd) as f:
        xyz2 = f.read()[0]
    xyz3 = io.loadh(get_fn('frame0.binpos.h5'), 'xyz')

    assert eq(xyz[1:], xyz2)
    assert eq(xyz, xyz3)


def test_write_1(tmpdir):
    xyz = np.array(np.random.randn(500, 10, 3), dtype=np.float32)
    fn = '{}/x.binpos'.format(tmpdir)
    with BINPOSTrajectoryFile(fn, 'w') as f:
        f.write(xyz)

    xyz2 = BINPOSTrajectoryFile(fn).read()
    assert eq(xyz, xyz2)


def test_write_2(tmpdir):
    xyz = np.array(np.random.randn(500, 10, 3), dtype=np.float32)
    fn = '{}/x.binpos'.format(tmpdir)
    with BINPOSTrajectoryFile(fn, 'w') as f:
        f.write(xyz[0:250])
        f.write(xyz[250:])

    xyz2 = BINPOSTrajectoryFile(fn).read()
    assert eq(xyz, xyz2)


def test_tell(get_fn):
    with BINPOSTrajectoryFile(get_fn('frame0.binpos')) as f:
        eq(f.tell(), 0)

        f.read(101)
        eq(f.tell(), 101)

        f.read(3)
        eq(f.tell(), 104)


def test_seek(get_fn):
    reference = BINPOSTrajectoryFile(get_fn('frame0.binpos')).read()
    with BINPOSTrajectoryFile(get_fn('frame0.binpos')) as f:
        xyz = f.read(1)[0]
        eq(xyz, reference[0])
        eq(f.tell(), 1)

        xyz = f.read(1)[0]
        eq(xyz, reference[1])
        eq(f.tell(), 2)

        f.seek(0)
        eq(f.tell(), 0)
        xyz = f.read(1)[0]
        eq(f.tell(), 1)
        eq(xyz, reference[0])

        f.seek(5)
        eq(f.read(1)[0], reference[5])
        eq(f.tell(), 6)

        f.seek(-5, 1)
        eq(f.tell(), 1)
        xyz = f.read(1)[0]
        eq(xyz, reference[1])

        f.seek(0, 2)
        eq(f.tell(), len(reference))



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2017 Stanford University and the Authors
#
# Authors: Robert McGibbon
# Contributors: Kyle A Beauchamp, Matthew Harrigan
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################

import sys
import functools
from pathlib import Path
from mdtraj.testing import eq
import numpy as np
import mdtraj as md
import mdtraj.utils
from mdtraj.utils import six
from mdtraj.core import element
import mdtraj.core.trajectory
import pytest
import mdtraj.formats
from collections import namedtuple

on_win = (sys.platform == 'win32')
on_py3 = (sys.version_info >= (3, 0))

TrajObj = namedtuple('TrajObj', ['fobj', 'fext', 'fn'])
file_objs = [
    (md.formats.NetCDFTrajectoryFile, 'nc'),
    (md.formats.HDF5TrajectoryFile, 'h5'),
    (md.formats.XTCTrajectoryFile, 'xtc'),
    (md.formats.TRRTrajectoryFile, 'trr'),
    (md.formats.DCDTrajectoryFile, 'dcd'),
    (md.formats.MDCRDTrajectoryFile, 'mdcrd'),
    (md.formats.BINPOSTrajectoryFile, 'binpos'),
    (md.formats.DTRTrajectoryFile, 'dtr'),
    (md.formats.XYZTrajectoryFile, 'xyz'),
    (md.formats.XYZTrajectoryFile, 'xyz.gz'),
    (md.formats.LAMMPSTrajectoryFile, 'lammpstrj'),
    (md.formats.TNGTrajectoryFile, 'tng'),
    (md.formats.LH5TrajectoryFile, 'lh5'),
    (md.formats.PDBTrajectoryFile, 'pdb'),
    (md.formats.PDBTrajectoryFile, 'pdb.gz'),
    (md.formats.AmberNetCDFRestartFile, 'ncrst'),
    (md.formats.AmberRestartFile, 'rst7'),
    (md.formats.GroTrajectoryFile, 'gro')
]


@pytest.fixture(params=file_objs, ids=lambda x: x[1])
def ref_traj(request):
    fobj, fext = request.param
    if (on_win and on_py3):
        if fext == 'lh5':
            pytest.skip("lh5 is not supported on Windows + Py3")

    if fext in ['ncrst', 'rst7']:
        pytest.xfail("These reference files don't exist yet")

    return TrajObj(fobj, fext, "frame0.{}".format(fext))


@pytest.fixture(params=file_objs, ids=lambda x: x[1])
def write_traj(request, tmpdir):
    fobj, fext = request.param
    if (on_win and on_py3):
        if fext == 'lh5':
            pytest.skip("lh5 is not supported on Windows + Py3")

    return TrajObj(fobj, fext, "{}/traj.{}".format(tmpdir, fext))


@pytest.fixture
def write_traj_with_box(write_traj):
    if write_traj.fext in ['binpos', 'xyz', 'xyz.gz', 'pdb', 'pdb.gz', 'lh5']:
        pytest.skip("{} does not store box information".format(write_traj.fext))
    else:
        return write_traj


def test_mismatch(get_fn):
    # loading a 22 atoms xtc with a topology that has 2,000 atoms
    # some kind of error should happen
    with pytest.raises(ValueError):
        md.load(get_fn('frame0.xtc'), top=get_fn('4ZUO.pdb'))


def test_box(get_fn):
    t = md.load(get_fn('native.pdb'))
    assert eq(t.unitcell_vectors, None)
    assert eq(t.unitcell_lengths, None)
    assert eq(t.unitcell_angles, None)
    assert eq(t.unitcell_volumes, None)

    t.unitcell_vectors = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]]).reshape(1, 3, 3)
    assert eq(np.array([1.0, 1.0, 1.0]), t.unitcell_lengths[0])
    assert eq(np.array([90.0, 90.0, 90.0]), t.unitcell_angles[0])
    assert eq(np.array([1.0]), t.unitcell_volumes)


def test_load_pdb_box(get_fn):
    t = md.load(get_fn('native2.pdb'), no_boxchk=True)
    assert eq(t.unitcell_lengths[0], np.array([0.1, 0.2, 0.3]))
    assert eq(t.unitcell_angles[0], np.array([90.0, 90.0, 90.0]))
    assert eq(t.unitcell_vectors[0], np.array([[0.1, 0, 0], [0, 0.2, 0], [0, 0, 0.3]]))


def test_load_pdb_gz(get_fn):
    t = md.load(get_fn('1ncw.pdb.gz'))
    assert eq(t.n_atoms, 3990)


def test_box_load_save(write_traj_with_box, get_fn):
    t = md.load(get_fn('native2.pdb'), no_boxchk=True)
    top = md.load_topology(get_fn('native.pdb'), no_boxchk=True)

    # make sure than through a load/save
    # cycle, the box information is preserved:
    t.save(write_traj_with_box.fn)
    t2 = md.load(write_traj_with_box.fn, top=top)

    assert t.unitcell_vectors is not None
    assert eq(t.xyz, t2.xyz, decimal=3)
    assert eq(t.unitcell_vectors, t2.unitcell_vectors)
    assert eq(t.unitcell_angles, t2.unitcell_angles)
    assert eq(t.unitcell_lengths, t2.unitcell_lengths)


def test_slice(get_fn):
    t = md.load(get_fn('traj.h5'))

    # with copying
    assert eq((t[0:5] + t[5:10]).xyz, t[0:10].xyz)
    assert eq((t[0:5] + t[5:10]).time, t[0:10].time)
    assert eq((t[0:5] + t[5:10]).unitcell_vectors, t[0:10].unitcell_vectors)
    assert eq((t[0:5] + t[5:10]).unitcell_lengths, t[0:10].unitcell_lengths)
    assert eq((t[0:5] + t[5:10]).unitcell_angles, t[0:10].unitcell_angles)

    # without copying (in place)
    assert eq((t.slice(key=range(5), copy=False) + t.slice(key=range(5, 10), copy=False)).xyz,
              t.slice(key=range(10), copy=False).xyz)
    assert eq((t.slice(key=range(5), copy=False) + t.slice(key=range(5, 10), copy=False)).time,
              t.slice(key=range(10), copy=False).time)
    assert eq((t.slice(key=range(5), copy=False) + t.slice(key=range(5, 10), copy=False)).unitcell_vectors,
              t.slice(key=range(10), copy=False).unitcell_vectors)
    assert eq((t.slice(key=range(5), copy=False) + t.slice(key=range(5, 10), copy=False)).unitcell_lengths,
              t.slice(key=range(10), copy=False).unitcell_lengths)
    assert eq((t.slice(key=range(5), copy=False) + t.slice(key=range(5, 10), copy=False)).unitcell_angles,
              t.slice(key=range(10), copy=False).unitcell_angles)


def test_slice2(get_fn):
    t = md.load(get_fn('traj.h5'))
    # with copying
    assert t[0] == t[[0, 1]][0]
    # without copying (in place)
    assert t.slice(key=0, copy=False) == t.slice(key=[0, 1], copy=True)[0]


def has_time_info(fext):
    # Some formats don't save time information
    return fext not in ['dcd', 'binpos', 'pdb', 'pdb.gz', 'xyz', 'xyz.gz', 'lammpstrj', 'lh5', 'mdcrd']


def precision(fext):
    if fext in ['xyz', 'lammpstrj', 'lh5']:
        return 3
    else:
        return 6


def precision2(fext1, fext2):
    return min(precision(fext1), precision(fext2))


def test_read_path(ref_traj, get_fn):
    top = get_fn('native.pdb')
    t = md.load(Path(get_fn(ref_traj.fn)), top=top)


def test_write_path(write_traj, get_fn):
    if write_traj.fext in ('ncrst', 'rst7'):
        pytest.skip("{} can only store 1 frame per file".format(write_traj.fext))
    if write_traj.fext in ('mdcrd'):
        pytest.skip("{} can only store rectilinear boxes".format(write_traj.fext))
    t = md.load(get_fn('traj.h5'))
    if t.unitcell_vectors is None:
        if write_traj.fext in ('dtr', 'lammpstrj'):
            pytest.skip("{} needs to write unitcells".format(write_traj.fext))
    t.save(Path(write_traj.fn))


def test_read_write(ref_traj, write_traj, get_fn):
    if write_traj.fext in ('ncrst', 'rst7'):
        pytest.skip("{} can only store 1 frame per file".format(write_traj.fext))
    if write_traj.fext in ('mdcrd'):
        pytest.skip("{} can only store rectilinear boxes".format(write_traj.fext))

    top = get_fn('native.pdb')
    t = md.load(get_fn(ref_traj.fn), top=top)

    if t.unitcell_vectors is None:
        if write_traj.fext in ('dtr', 'lammpstrj'):
            pytest.skip("{} needs to write unitcells".format(write_traj.fext))

    t.save(write_traj.fn)
    t2 = md.load(write_traj.fn, top=top)
    eq(t.xyz, t2.xyz, decimal=precision2(ref_traj.fext, write_traj.fext))
    if has_time_info(write_traj.fext):
        eq(t.time, t2.time, decimal=3)


def test_load(ref_traj, get_fn):
    nat = md.load(get_fn('native.pdb'))
    num_block = 3
    t0 = md.load(get_fn(ref_traj.fn), top=nat, discard_overlapping_frames=True)
    t1 = md.load(get_fn(ref_traj.fn), top=nat, discard_overlapping_frames=False)
    t2 = md.load([get_fn(ref_traj.fn) for _ in range(num_block)], top=nat, discard_overlapping_frames=False)
    t3 = md.load([get_fn(ref_traj.fn) for _ in range(num_block)], top=nat, discard_overlapping_frames=True)

    # these don't actually overlap, so discard_overlapping_frames should
    # have no effect. the overlap is between the last frame of one and the
    # first frame of the next.
    assert eq(t0.n_frames, t1.n_frames)
    assert eq(t0.n_frames * num_block, t2.n_frames)
    assert eq(t3.n_frames, t2.n_frames)


def test_hdf5(get_fn):
    t = md.load(get_fn('traj.h5'))
    t2 = md.load(get_fn('native.pdb'))
    t3 = md.load(get_fn('traj.h5'), frame=8)

    assert t.topology == t2.topology
    assert eq(t.time, 0.002 * (1 + np.arange(100)))
    assert eq(t.time, 0.002 * (1 + np.arange(100)))
    assert eq(t[8].xyz, t3.xyz)
    assert eq(t[8].time, t3.time)
    assert eq(t[8].unitcell_vectors, t3.unitcell_vectors)


def test_center(get_fn):
    traj = md.load(get_fn('traj.h5'))
    traj.center_coordinates()
    mu = traj.xyz.mean(1)
    mu0 = np.zeros(mu.shape)
    eq(mu0, mu)

    for a in traj.top.atoms:
        # Set all masses equal so we can compare against unweighted result
        a.element = element.hydrogen

    traj.center_coordinates(mass_weighted=True)
    mu2 = traj.xyz.mean(1)
    eq(mu0, mu2)


def test_center_aind(get_fn):
    traj = md.load(get_fn('traj.h5'))
    traj.restrict_atoms(np.arange(0, traj.n_atoms, 2))
    traj.center_coordinates()
    mu = traj.xyz.mean(1)
    mu0 = np.zeros(mu.shape)
    eq(mu0, mu)

    for a in traj.top.atoms:
        # Set all masses equal so we can compare against unweighted result
        a.element = element.hydrogen

    traj.center_coordinates(mass_weighted=True)
    mu2 = traj.xyz.mean(1)
    eq(mu0, mu2)


def test_float_atom_indices_exception(ref_traj, get_fn):
    # Is an informative error message given when you supply floats for atom_indices?
    top = md.load(get_fn('native.pdb')).topology

    try:
        md.load(get_fn(ref_traj.fn), atom_indices=[0.5, 1.3], top=top)
    except ValueError as e:
        if six.PY3:
            assert e.args[0] == 'indices must be of an integer type. float64 is not an integer type'
        else:
            assert e.message == 'indices must be of an integer type. float64 is not an integer type'


def test_restrict_atoms(get_fn):
    traj = md.load(get_fn('traj.h5'))
    time_address = traj.time.ctypes.data

    desired_atom_indices = [0, 1, 2, 5]
    traj.restrict_atoms(desired_atom_indices)
    atom_indices = [a.index for a in traj.top.atoms]
    eq([0, 1, 2, 3], atom_indices)
    eq(traj.xyz.shape[1], 4)
    eq(traj.n_atoms, 4)
    eq(traj.n_residues, 1)
    eq(len(traj.top._bonds), 2)
    eq(traj.n_residues, traj.topology._numResidues)
    eq(traj.n_atoms, traj.topology._numAtoms)
    eq(np.array([a.index for a in traj.topology.atoms]), np.arange(traj.n_atoms))

    # assert that the time field was not copied
    assert traj.time.ctypes.data == time_address


def test_restrict_atoms_not_inplace(get_fn):
    traj = md.load(get_fn('traj.h5'))
    traj_backup = md.load(get_fn('traj.h5'))
    desired_atom_indices = [0, 1, 2, 5]

    sliced = traj.restrict_atoms(desired_atom_indices, inplace=False)

    # make sure the original one was not modified
    eq(traj.xyz, traj_backup.xyz)
    eq(traj.topology, traj_backup.topology)

    eq(list(range(4)), [a.index for a in sliced.top.atoms])
    eq(sliced.xyz.shape[1], 4)
    eq(sliced.n_atoms, 4)
    eq(sliced.n_residues, 1)
    eq(len(sliced.top._bonds), 2)
    eq(sliced.n_residues, sliced.topology._numResidues)
    eq(sliced.n_atoms, sliced.topology._numAtoms)
    eq(np.array([a.index for a in sliced.topology.atoms]), np.arange(sliced.n_atoms))

    # make sure the two don't alias the same memory
    assert traj.time.ctypes.data != sliced.time.ctypes.data
    assert traj.unitcell_angles.ctypes.data != sliced.unitcell_angles.ctypes.data
    assert traj.unitcell_lengths.ctypes.data != sliced.unitcell_lengths.ctypes.data


def test_array_vs_matrix(get_fn):
    top = md.load(get_fn('native.pdb')).topology
    xyz = np.random.randn(1, 22, 3)
    xyz_mat = np.matrix(xyz)
    t1 = md.Trajectory(xyz, top)
    t2 = md.Trajectory(xyz_mat, top)

    eq(t1.xyz, xyz)
    eq(t2.xyz, xyz)


def test_pdb_unitcell_loadsave(tmpdir, get_fn):
    # Make sure that nonstandard unitcell dimensions are saved and loaded
    # correctly with PDB
    tref = md.load(get_fn('native.pdb'))
    tref.unitcell_lengths = 1 + 0.1 * np.random.randn(tref.n_frames, 3)
    tref.unitcell_angles = 90 + 0.0 * np.random.randn(tref.n_frames, 3)
    fn = "{}/x.pdb".format(tmpdir)
    tref.save(fn)

    tnew = md.load(fn)
    eq(tref.unitcell_vectors, tnew.unitcell_vectors, decimal=3)


def test_load_combination(ref_traj, get_fn):
    # Test that the load function's stride and atom_indices work across
    # all trajectory formats

    topology = md.load(get_fn('native.pdb')).topology
    ainds = np.array([a.index for a in topology.atoms if a.element.symbol == 'C'])

    no_kwargs = md.load(get_fn(ref_traj.fn), top=topology)
    strided3 = md.load(get_fn(ref_traj.fn), top=topology, stride=3)
    subset = md.load(get_fn(ref_traj.fn), top=topology, atom_indices=ainds)

    # test 1
    t1 = no_kwargs
    t2 = strided3
    assert eq(t1.xyz[::3], t2.xyz)
    assert eq(t1.time[::3], t2.time)
    if t1.unitcell_vectors is not None:
        assert eq(t1.unitcell_vectors[::3], t2.unitcell_vectors)
    assert eq(t1.topology, t2.topology)

    # test 2
    t1 = no_kwargs
    t2 = subset
    assert eq(t1.xyz[:, ainds, :], t2.xyz)
    assert eq(t1.time, t2.time)
    if t1.unitcell_vectors is not None:
        assert eq(t1.unitcell_vectors, t2.unitcell_vectors)
    assert eq(t1.topology.subset(ainds), t2.topology)


def test_no_topology():
    # We can make trajectories without a topology
    md.Trajectory(xyz=np.random.randn(10, 5, 3), topology=None)


def test_join():
    xyz = np.random.rand(10, 5, 3)
    # overlapping frames
    t1 = md.Trajectory(xyz=xyz[:5], topology=None)
    t2 = md.Trajectory(xyz=xyz[4:], topology=None)

    t3 = t1.join(t2, discard_overlapping_frames=True)
    t4 = t1.join(t2, discard_overlapping_frames=False)
    eq(t3.xyz, xyz)
    eq(len(t4.xyz), 11)
    eq(t4.xyz, np.vstack((xyz[:5], xyz[4:])))


def test_md_join(get_fn):
    fn = get_fn('traj.h5')
    t_ref = md.load(get_fn('frame0.h5'))[:20]
    loaded = md.load(fn, top=t_ref, stride=2)
    iterloaded = md.join(md.iterload(fn, top=t_ref, stride=2, chunk=6))
    eq(loaded.xyz, iterloaded.xyz)
    eq(loaded.time, iterloaded.time)
    eq(loaded.unitcell_angles, iterloaded.unitcell_angles)
    eq(loaded.unitcell_lengths, iterloaded.unitcell_lengths)


def test_stack_1(get_fn):
    t1 = md.load(get_fn('native.pdb'))
    t2 = t1.stack(t1)
    eq(t2.n_atoms, 2 * t1.n_atoms)
    eq(t2.topology._numAtoms, 2 * t1.n_atoms)
    eq(t1.xyz, t2.xyz[:, 0:t1.n_atoms])
    eq(t1.xyz, t2.xyz[:, t1.n_atoms:])


def test_stack_2():
    t1 = md.Trajectory(xyz=np.random.rand(10, 5, 3), topology=None)
    t2 = md.Trajectory(xyz=np.random.rand(10, 6, 3), topology=None)
    t3 = t1.stack(t2)

    eq(t3.xyz[:, :5], t1.xyz)
    eq(t3.xyz[:, 5:], t2.xyz)
    eq(t3.n_atoms, 11)


def test_seek_read_mode(ref_traj, get_fn):
    # Test the seek/tell capacity of the different TrajectoryFile objects in
    # read mode. Basically, we just seek around the files and read different
    # segments, keeping track of our location manually and checking with both
    # tell() and by checking that the right coordinates are actually returned
    fobj = ref_traj.fobj
    fn = ref_traj.fn

    if ref_traj.fobj is md.formats.PDBTrajectoryFile:
        pytest.xfail("PDB Files don't support seeking")
    if ref_traj.fext == 'xyz.gz':
        pytest.xfail("This is broken")
    if ref_traj.fext == 'gro':
        pytest.xfail("This is broken")

    point = 0
    xyz = md.load(get_fn(fn), top=get_fn('native.pdb')).xyz
    length = len(xyz)
    kwargs = {}
    if fobj is md.formats.MDCRDTrajectoryFile:
        kwargs = {'n_atoms': 22}

    with fobj(get_fn(fn), **kwargs) as f:
        for i in range(100):
            r = np.random.rand()
            if r < 0.25:
                offset = np.random.randint(-5, 5)
                if 0 < point + offset < length:
                    point += offset
                    f.seek(offset, 1)
                else:
                    f.seek(0)
                    point = 0
            if r < 0.5:
                offset = np.random.randint(1, 10)
                if point + offset < length:
                    read = f.read(offset)
                    if fobj not in [md.formats.BINPOSTrajectoryFile, md.formats.LH5TrajectoryFile,
                                    md.formats.XYZTrajectoryFile]:
                        read = read[0]
                    readlength = len(read)
                    read = mdtraj.utils.in_units_of(read, f.distance_unit, 'nanometers')
                    eq(xyz[point:point + offset], read)
                    point += readlength
            elif r < 0.75:
                offset = np.random.randint(low=-100, high=0)
                try:
                    f.seek(offset, 2)
                    point = length + offset
                except NotImplementedError:
                    # not all of the *TrajectoryFiles currently support
                    # seeking from the end, so we'll let this pass if they
                    # say that they dont implement this.
                    pass
            else:
                offset = np.random.randint(100)
                f.seek(offset, 0)
                point = offset

            eq(f.tell(), point)


def test_load_frame(ref_traj, get_fn):
    if ref_traj.fobj is md.formats.GroTrajectoryFile:
        pytest.xfail("Gro doesn't implement seek")
    trajectory = md.load(get_fn(ref_traj.fn), top=get_fn('native.pdb'))
    rand = np.random.randint(len(trajectory))
    frame = md.load_frame(get_fn(ref_traj.fn), index=rand, top=get_fn('native.pdb'))

    if ref_traj.fobj is md.formats.DTRTrajectoryFile:
        pytest.xfail("DTR doesn't load a single frame properly")
    eq(trajectory[rand].xyz, frame.xyz)
    eq(trajectory[rand].unitcell_vectors, frame.unitcell_vectors)
    if has_time_info(ref_traj.fext):
        eq(trajectory[rand].time, frame.time)


def test_load_frame_2eqq(get_fn):
    t1 = md.load(get_fn('2EQQ.pdb'))
    r = np.random.randint(len(t1))
    t2 = md.load_frame(get_fn('2EQQ.pdb'), r)
    eq(t1[r].xyz, t2.xyz)


def test_iterload(write_traj, get_fn):
    if write_traj.fext == 'dtr':
        pytest.xfail("This is broken with dtr")
    t_ref = md.load(get_fn('frame0.h5'))[:20]

    if write_traj.fext in ('ncrst', 'rst7'):
        pytest.skip("Only 1 frame per file format")

    t_ref.save(write_traj.fn)

    for stride in [1, 2, 3]:
        loaded = md.load(write_traj.fn, top=t_ref, stride=stride)
        iterloaded = functools.reduce(lambda a, b: a.join(b),
                                      md.iterload(write_traj.fn, top=t_ref, stride=stride, chunk=6))
        eq(loaded.xyz, iterloaded.xyz)
        eq(loaded.time, iterloaded.time)
        eq(loaded.unitcell_angles, iterloaded.unitcell_angles)
        eq(loaded.unitcell_lengths, iterloaded.unitcell_lengths)


def test_iterload_skip(ref_traj, get_fn):
    if ref_traj.fobj is md.formats.PDBTrajectoryFile:
        pytest.xfail("PDB Iterloads an extra frame!!")
    if ref_traj.fobj is md.formats.GroTrajectoryFile:
        pytest.xfail("Not implemented for some reason")
    if ref_traj.fext in ('ncrst', 'rst7'):
        pytest.skip("Only 1 frame per file format")

    top = md.load(get_fn('native.pdb'))
    t_ref = md.load(get_fn(ref_traj.fn), top=top)

    for cs in [0, 1, 11, 100]:
        for skip in [0, 1, 20, 101]:
            t = functools.reduce(lambda a, b: a.join(b),
                                 md.iterload(get_fn(ref_traj.fn), skip=skip, top=top, chunk=cs))
            eq(t_ref.xyz[skip:], t.xyz)
            eq(t_ref.time[skip:], t.time)
            eq(t_ref.topology, t.topology)


def test_iterload_chunk_dcd(get_fn):
    # Makes sure that the actual chunk size yielded by iterload corresponds to the number of
    # frames specified when calling it (for dcd files).
    file = get_fn("alanine-dipeptide-explicit.dcd")
    top = get_fn("alanine-dipeptide-explicit.pdb")

    skip_frames = 3
    frames_chunk = 2

    full = md.load(file, top=top, stride=skip_frames)
    length = len(full)
    

    chunks = []
    for traj_chunk in md.iterload(file, top=top, stride=skip_frames, chunk=frames_chunk):        
        chunks.append(traj_chunk)
    joined = md.join(chunks)
    assert len(full) == len(joined)
    assert eq(full.xyz, joined.xyz)


def test_save_load(write_traj, get_fn):
    # this cycles all the known formats you can save to, and then tries
    # to reload, using just a single-frame file.

    t_ref = md.load(get_fn('native.pdb'))
    t_ref.unitcell_vectors = np.array([[[1, 0, 0], [0, 1, 0], [0, 0, 1]]])

    t_ref.save(write_traj.fn)
    t = md.load(write_traj.fn, top=t_ref.topology)

    eq(t.xyz, t_ref.xyz)
    eq(t.time, t_ref.time)
    if t._have_unitcell:
        eq(t.unitcell_angles, t_ref.unitcell_angles)
        eq(t.unitcell_lengths, t_ref.unitcell_lengths)


def test_force_overwrite(write_traj, get_fn):
    if write_traj.fext == 'dtr':
        pytest.xfail("This is broken with dtr")

    t_ref = md.load(get_fn('native2.pdb'), no_boxchk=True)
    open(write_traj.fn, 'w').close()
    t_ref.save(write_traj.fn, force_overwrite=True)


def test_force_noverwrite(write_traj, get_fn):
    t_ref = md.load(get_fn('native2.pdb'), no_boxchk=True)
    open(write_traj.fn, 'w').close()
    with pytest.raises(IOError):
        t_ref.save(write_traj.fn, force_overwrite=False)


def test_open_and_load(get_fn):
    # These aren't tested in test_length because they don't support length!
    files = ['frame0.mdcrd', '4waters.arc', 'frame0.lammpstrj']

    for file in files:
        if file.endswith('.mdcrd'):
            opened = md.open(get_fn(file), n_atoms=22)
        else:
            opened = md.open(get_fn(file))

        loaded = md.load(get_fn(file), top=get_fn('native.pdb'))


def test_length(get_fn):
    files = ['frame0.nc', 'frame0.h5', 'frame0.xtc', 'frame0.trr',
             'frame0.dcd', '2EQQ.pdb',
             'frame0.binpos', 'frame0.xyz', 'frame0.tng']
    if not (on_win and on_py3):
        files.append('frame0.lh5')

    for file in files:
        opened = md.open(get_fn(file))

        if '.' + file.rsplit('.', 1)[-1] in mdtraj.core.trajectory._TOPOLOGY_EXTS:
            top = file
        else:
            top = 'native.pdb'

        loaded = md.load(get_fn(file), top=get_fn(top))
        assert len(opened) == len(loaded)


def test_unitcell(write_traj, get_fn):
    # make sure that bogus unitcell vecotrs are not saved

    if write_traj.fext in ['rst7', 'ncrst', 'lammpstrj', 'dtr']:
        pytest.xfail('{} seems to need unit vectors'.format(write_traj.fext))

    top = md.load(get_fn('native.pdb')).restrict_atoms(range(5)).topology
    t = md.Trajectory(xyz=np.random.randn(100, 5, 3), topology=top)
    t.save(write_traj.fn)
    assert eq(md.load(write_traj.fn, top=top).unitcell_vectors, None)


def test_chunk0_iterload(get_fn):
    filename = 'frame0.h5'

    trj0 = md.load(get_fn(filename))

    for trj in md.iterload(get_fn(filename), chunk=0):
        pass

    eq(trj0.n_frames, trj.n_frames)


def test_hashing(get_fn):
    frames = [frame for frame in
              md.iterload(get_fn('frame0.xtc'), chunk=1,
                          top=get_fn('native.pdb'))]
    hashes = [hash(frame) for frame in frames]
    # check all frames have a unique hash value
    assert len(hashes) == len(set(hashes))

    # change topology and ensure hash changes too
    top = frames[0].topology
    top.add_bond(top.atom(0), top.atom(1))

    last_frame_hash = hash(frames[0])
    assert last_frame_hash != hashes[-1]


def test_smooth(get_fn):
    from scipy.signal import lfilter, lfilter_zi, filtfilt, butter

    pad = 5
    order = 3
    b, a = butter(order, 2.0 / pad)
    zi = lfilter_zi(b, a)

    signal = np.sin(np.arange(100))
    padded = np.r_[signal[pad - 1: 0: -1], signal, signal[-1: -pad: -1]]

    z, _ = lfilter(b, a, padded, zi=zi * padded[0])
    z2, _ = lfilter(b, a, z, zi=zi * z[0])

    output = filtfilt(b, a, padded)
    test = np.loadtxt(get_fn('smooth.txt'))

    eq(output, test)


def test_image_molecules(get_fn):
    # Load trajectory with periodic box
    t = md.load(get_fn('alanine-dipeptide-explicit.dcd'), top=get_fn('alanine-dipeptide-explicit.pdb'))
    # Image to new trajectory
    t_new = t.image_molecules(inplace=False)
    # Image inplace without making molecules whole
    t.image_molecules(inplace=True, make_whole=False)
    # Image inplace with making molecules whole
    t.image_molecules(inplace=True, make_whole=True)
    # Image with specified anchor molecules
    molecules = t.topology.find_molecules()
    anchor_molecules = molecules[0:3]
    t.image_molecules(inplace=True, anchor_molecules=anchor_molecules)


def test_load_pdb_no_standard_names(get_fn):
    # Minimal test. Standard_names=False will force load_pdb.py
    # to NOT replace any non-standard atom or residue names in the topology
    md.load(get_fn('native2.pdb'), standard_names=False, no_boxchk=True)
    md.load_pdb(get_fn('native2.pdb'), standard_names=False, no_boxchk=True)


def test_load_with_atom_indices(get_fn):
    t1 = md.load(get_fn('frame0.xtc'), top=get_fn('frame0.gro'), atom_indices=[5])
    t2 = md.load(get_fn('frame0.xtc'), top=get_fn('frame0.gro'))
    t2 = t2.atom_slice([5])
    eq(t1.xyz, t2.xyz)
    eq(t1.time, t2.time)


def test_load_with_frame(get_fn):
    t1 = md.load(get_fn('frame0.xtc'), top=get_fn('frame0.pdb'), frame=3)
    t2 = md.load(get_fn('frame0.xtc'), top=get_fn('frame0.pdb'))
    t2 = t2.slice([3])
    eq(t1.xyz, t2.xyz)
    eq(t1.time, t2.time)


def test_add_remove_atoms(get_fn):
    t = md.load(get_fn('aaqaa-wat.pdb'))
    top = t.topology
    old_atoms = list(top.atoms)[:]
    # Add an atom 'MW' at the end of each water molecule
    for r in list(top.residues)[::-1]:
        if r.name != 'HOH': continue
        atoms = list(r.atoms)
        midx = atoms[-1].index+1
        top.insert_atom('MW',None,r,index=midx)
    mwidx = [a.index for a in list(top.atoms) if a.name == 'MW']
    # Check to see whether the 'MW' atoms have the correct index
    assert mwidx == [183+4*i for i in range(83)]
    # Now delete the atoms again
    for r in list(top.residues)[::-1]:
        if r.name != 'HOH': continue
        atoms = list(r.atoms)
        top.delete_atom_by_index(atoms[-1].index)
    roundtrip_atoms = list(top.atoms)[:]
    # Ensure the atoms are the same after a round trip of adding / deleting
    assert old_atoms == roundtrip_atoms



import os
import sys
import mdtraj


def test_1():
    capi = mdtraj.capi()
    assert os.path.isdir(capi['lib_dir'])
    assert os.path.isdir(capi['include_dir'])
    if sys.platform == 'win32':
        assert os.path.isfile(os.path.join(capi['lib_dir'], 'libtheobald.lib'))
    else:
        assert os.path.isfile(os.path.join(capi['lib_dir'], 'libtheobald.a'))



import os
import os.path
import numpy as np
import tempfile
import mdtraj as md
import pytest


def test_1(get_fn):
    # https://github.com/mdtraj/mdtraj/issues/438
    try:
        dir = tempfile.mkdtemp()
        fn = os.path.join(dir, 'temp.npy')
        traj = md.load(get_fn('frame0.h5'))
        np.save(fn, traj.xyz)

        traj.xyz = np.load(fn, mmap_mode='r')

        # since traj isn't precentered, this requires centering
        # the coordinates which is done inplace. but that's not possible
        # with mmap_mode = 'r'
        with pytest.raises(ValueError):
            md.rmsd(traj, traj, 0)

        # this should work
        traj.xyz = np.load(fn, mmap_mode='c')
        md.rmsd(traj, traj, 0)

    finally:
        del traj
        os.unlink(fn)
        os.rmdir(dir)


def test_2(get_fn):
    # https://github.com/mdtraj/mdtraj/issues/438
    try:
        dir = tempfile.mkdtemp()
        fn = os.path.join(dir, 'temp.npy')
        traj = md.load(get_fn('frame0.h5'))
        # precenter the coordinates
        traj.center_coordinates()
        traces = traj._rmsd_traces
        np.save(fn, traj.xyz)
        traj.xyz = np.load(fn, mmap_mode='r')
        traj._rmsd_traces = traces

        with pytest.raises(ValueError):
            md.rmsd(traj, traj, 0, precentered=True)

    finally:
        del traj
        os.unlink(fn)
        os.rmdir(dir)



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2013 Stanford University and the Authors
#
# Authors: Robert McGibbon
# Contributors: Teng Lin
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################


import numpy as np
from mdtraj.formats import DTRTrajectoryFile, DCDTrajectoryFile
from mdtraj.testing import eq
import pytest

def test_read(get_fn):
    # Test the default read and compare against reference trajectory in dcd format
    fn_dtr = get_fn('frame0.dtr')
    fn_dcd = get_fn('frame0.dcd')
    dtr_traj = DTRTrajectoryFile(fn_dtr)
    eq(len(dtr_traj), 501)
    xyz, times, cell_lens, cell_angles = dtr_traj.read()
    xyz2, cell_lens2, cell_angles2 = DCDTrajectoryFile(fn_dcd).read()
    eq(xyz, xyz2)
    eq(cell_lens, cell_lens2)
    eq(cell_angles, cell_angles2)


def test_read_1(get_fn):
    # test read with n_frame
    fn_dtr = get_fn('frame0.dtr')
    xyz, times, cell_lens, cell_angles = DTRTrajectoryFile(fn_dtr).read()
    xyz2, times2, cell_lens2, cell_angles2 = DTRTrajectoryFile(fn_dtr).read(n_frames=501)
    eq(xyz, xyz2)
    eq(times, times2)
    eq(cell_lens, cell_lens2)
    eq(cell_angles, cell_angles2)


def test_read_2(get_fn):
    # test read with atom indices
    fn_dtr = get_fn('frame0.dtr')
    indices = np.array([0, 3, 12, 4])
    xyz, times, cell_lens, cell_angles = DTRTrajectoryFile(fn_dtr).read()
    xyz2, times2, cell_lens2, cell_angles2 = DTRTrajectoryFile(fn_dtr).read(atom_indices=indices)
    eq(xyz[:, indices, :], xyz2)
    eq(times, times2)
    eq(cell_lens, cell_lens2)
    eq(cell_angles, cell_angles2)


def test_read_3(get_fn):
    # Test read with n_frames
    fn_dtr = get_fn('frame0.dtr')
    dtr_traj = DTRTrajectoryFile(fn_dtr)
    dtr_traj.seek(1)
    xyz, times, cell_lens, cell_angles = dtr_traj.read(n_frames=900)
    eq(len(xyz), 500)


def test_read_stride(get_fn):
    # Read dtr with stride
    fn_dtr = get_fn('frame0.dtr')
    with DTRTrajectoryFile(fn_dtr) as f:
        xyz1, times1, box_lengths1, box_angles1 = f.read()
    with DTRTrajectoryFile(fn_dtr) as f:
        xyz2, times2, box_lengths2, box_angles2 = f.read(stride=2)

    assert eq(xyz1[::2], xyz2)
    assert eq(times1[::2], times2)
    assert eq(box_lengths1[::2], box_lengths2)
    assert eq(box_angles1[::2], box_angles2)


def test_read_4(get_fn):
    # Read dtr with stride and n_frames
    fn_dtr = get_fn('frame0.dtr')
    with DTRTrajectoryFile(fn_dtr) as f:
        xyz1, times1, box_lengths1, box_angles1 = f.read()
    with DTRTrajectoryFile(fn_dtr) as f:
        xyz2, times2, box_lengths2, box_angles2 = f.read(n_frames=300, stride=2)

    assert eq(xyz1[::2], xyz2)
    assert eq(times1[::2], times2)
    assert eq(box_lengths1[::2], box_lengths2)
    assert eq(box_angles1[::2], box_angles2)


def test_read_5(get_fn):
    # Check streaming read of frames 1 at a time
    fn_dtr = get_fn('frame0.dtr')
    xyz_ref, times_ref, box_lengths_ref, box_angles_ref = DTRTrajectoryFile(fn_dtr).read()

    reader = DTRTrajectoryFile(fn_dtr)
    for i in range(len(xyz_ref)):
        xyz, times, box_lenths, box_angles = reader.read(1)
        eq(xyz_ref[np.newaxis, i], xyz)
        eq(times_ref[np.newaxis, i], times)
        eq(box_lengths_ref[np.newaxis, i], box_lenths)
        eq(box_angles_ref[np.newaxis, i], box_angles)


def test_read_6(get_fn):
    # Check streaming read followed by reading the 'rest'
    fn_dtr = get_fn('frame0.dtr')
    xyz_ref, times_ref, box_lengths_ref, box_angles_ref = DTRTrajectoryFile(fn_dtr).read()

    reader = DTRTrajectoryFile(fn_dtr)
    for i in range(len(xyz_ref) // 2):
        xyz, times, box_lenths, box_angles = reader.read(1)
        eq(xyz_ref[np.newaxis, i], xyz)
        eq(times_ref[np.newaxis, i], times)
        eq(box_lengths_ref[np.newaxis, i], box_lenths)
        eq(box_angles_ref[np.newaxis, i], box_angles)

    xyz_rest, times_rest, box_rest, angles_rest = reader.read()
    i = len(xyz_ref) // 2
    assert eq(xyz_ref[i:], xyz_rest)
    assert eq(times_ref[i:], times_rest)
    assert eq(box_lengths_ref[i:], box_rest)
    assert eq(box_angles_ref[i:], angles_rest)

    assert len(xyz_ref) == i + len(xyz_rest)


def test_read_7(get_fn):
    # Test two full reads
    fn_dtr = get_fn('frame0.dtr')
    reader = DTRTrajectoryFile(fn_dtr)
    xyz, times, cell_lens, cell_angles = reader.read()
    xyz, times, cell_lens, cell_angles = reader.read()
    eq(len(xyz), 0)
    eq(len(times), 0)
    eq(len(cell_lens), 0)
    eq(len(cell_angles), 0)


def test_read_8(get_fn):
    fn_dtr = get_fn('frame0.dtr')
    with DTRTrajectoryFile(fn_dtr) as f:
        xyz_ref, times_ref, box_lengths_ref, box_angles_ref = f.read()
    with DTRTrajectoryFile(fn_dtr) as f:
        xyz, times, box_lengths, box_angles = f.read(atom_indices=slice(None, None, 2))

    assert eq(xyz_ref[:, ::2, :], xyz)


def test_write_1(tmpdir, get_fn):
    # Test write
    fn_dtr = get_fn('frame0.dtr')
    fn = '{}/x.dtr'.format(tmpdir)
    xyz, times, cell_lens, cell_angles = DTRTrajectoryFile(fn_dtr).read()
    xyz += 1
    DTRTrajectoryFile(fn, 'w').write(xyz, cell_lengths=cell_lens,
                                     cell_angles=cell_angles, times=times)

    xyz2, times2, cell_lens2, cell_angles2 = DTRTrajectoryFile(fn).read()

    eq(xyz, xyz2)
    eq(times, times2)
    eq(cell_lens, cell_lens2)
    eq(cell_angles, cell_angles2)


def test_write_2(tmpdir, get_fn):
    # Test two separate write call
    fn_dtr = get_fn('frame0.dtr')
    fn = '{}/x.dtr'.format(tmpdir)
    xyz, times, cell_lens, cell_angles = DTRTrajectoryFile(fn_dtr).read()
    writer = DTRTrajectoryFile(fn, 'w')
    writer.write(xyz, cell_lengths=cell_lens,
                 cell_angles=cell_angles, times=times)

    n_frames = len(xyz)
    times += 50.0
    writer.write(xyz, cell_lengths=cell_lens,
                 cell_angles=cell_angles, times=times)

    writer.close()

    xyz2, times2, cell_lens2, cell_angles2 = DTRTrajectoryFile(fn).read()

    eq(len(xyz2), n_frames * 2)
    eq(xyz, xyz2[n_frames:])
    eq(times, times2[n_frames:])
    eq(cell_lens, cell_lens2[n_frames:])
    eq(cell_angles, cell_angles2[n_frames:])


def test_write_3(tmpdir):
    # Test a random write operation
    xyz = np.array(np.random.uniform(low=-50, high=-50, size=(3, 17, 3)), dtype=np.float32)
    times = np.array([1, 23.0, 48.0], dtype=np.float64)
    cell_lengths = np.array(np.random.uniform(low=100, high=200, size=(3, 3)), dtype=np.float32)
    cell_angles = np.array([[90, 90, 90],
                            [80, 100, 120],
                            [120, 90, 80]],
                           dtype=np.float32)

    fn = '{}/x.dtr'.format(tmpdir)
    with DTRTrajectoryFile(fn, 'w') as f:
        f.write(xyz, cell_lengths=cell_lengths,
                cell_angles=cell_angles, times=times)
    with DTRTrajectoryFile(fn) as f:
        xyz2, times2, cell_lengths2, cell_angles2 = f.read()

    eq(xyz, xyz2)


def test_write_4(tmpdir):
    # Test write error
    xyz = np.array(np.random.uniform(low=-50, high=-50, size=(3, 17, 3)), dtype=np.float32)
    times = np.array([1, 23.0, 48.0], dtype=np.float64)
    cell_lengths = np.array(np.random.uniform(low=100, high=200, size=(3, 3)), dtype=np.float32)
    cell_angles = np.array([[90, 90, 90],
                            [80, 100, 120],
                            [120, 90, 80]],
                           dtype=np.float32)

    bad_times = np.array([21, 3.0, 48.0], dtype=np.float64)
    fn = '{}/x.dtr'.format(tmpdir)
    f = DTRTrajectoryFile(fn, 'w')
    with pytest.raises(ValueError):
        f.write(xyz, cell_lengths=cell_lengths)
    with pytest.raises(ValueError):
        f.write(xyz, cell_angles=cell_angles)
    with pytest.raises(ValueError):
        f.write(xyz, times=times)
    with pytest.raises(ValueError):
        f.write(xyz, cell_lengths=cell_lengths,
                cell_angles=cell_angles, times=bad_times)
    f.close()


def test_seek(get_fn):
    fn_dtr = get_fn('frame0.dtr')
    reference = DTRTrajectoryFile(fn_dtr).read()[0]
    with DTRTrajectoryFile(fn_dtr) as f:
        eq(f.tell(), 0)
        eq(f.read(1)[0][0], reference[0])
        eq(f.tell(), 1)

        xyz = f.read(1)[0][0]
        eq(xyz, reference[1])
        eq(f.tell(), 2)

        f.seek(0)
        eq(f.tell(), 0)
        xyz = f.read(1)[0][0]
        eq(f.tell(), 1)
        eq(xyz, reference[0])

        f.seek(5)
        eq(f.read(1)[0][0], reference[5])
        eq(f.tell(), 6)

        f.seek(-5, 1)
        eq(f.tell(), 1)
        eq(f.read(1)[0][0], reference[1])


def test_read_closed(get_fn):
    fn_dtr = get_fn('frame0.dtr')
    with pytest.raises(IOError):
        f = DTRTrajectoryFile(fn_dtr)
        f.close()
        f.read()


def test_tell(get_fn):
    fn_dtr = get_fn('frame0.dtr')
    with DTRTrajectoryFile(fn_dtr) as f:
        last = len(f)
        eq(f.tell(), 0)

        f.read(2)
        eq(f.tell(), 2)

        f.read(100)
        eq(f.tell(), 102)

        f.seek(600)
        eq(f.tell(), last)




##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2013 Stanford University and the Authors
#
# Authors: Robert McGibbon
# Contributors:
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################

"""
This module implements the MDTraj HDF5 format described at
https://github.com/mdtraj/mdtraj/wiki/HDF5-Trajectory-Format
"""

##############################################################################
# Imports
##############################################################################

from __future__ import print_function, division
# stdlib
import os
import warnings
import inspect
from functools import wraps

import operator
from collections import namedtuple
try:
    import simplejson as json
except ImportError:
    import json

# 3rd party
import numpy as np

# ours
from mdtraj import version
import mdtraj.core.element as elem
from mdtraj.core.topology import Topology
from mdtraj.utils import in_units_of, ensure_type, import_, cast_indices
from mdtraj.utils.six import string_types
from mdtraj.formats.registry import FormatRegistry

__all__ = ['HDF5TrajectoryFile', 'load_hdf5']

Frames = namedtuple('Frames', ['coordinates', 'time', 'cell_lengths', 'cell_angles',
                               'velocities', 'kineticEnergy', 'potentialEnergy',
                               'temperature', 'alchemicalLambda'])

##############################################################################
# Code
##############################################################################

@FormatRegistry.register_loader('.h5')
@FormatRegistry.register_loader('.hdf5')
def load_hdf5(filename, stride=None, atom_indices=None, frame=None):
    """Load an MDTraj hdf5 trajectory file from disk.

    Parameters
    ----------
    filename : path-like
        Path of HDF Trajectory file.
    stride : int, default=None
        Only read every stride-th frame
    atom_indices : array_like, optional
        If not none, then read only a subset of the atoms coordinates from the
        file. This may be slightly slower than the standard read because it
        requires an extra copy, but will save memory.
    frame : int, optional
        Use this option to load only a single frame from a trajectory on disk.
        If frame is None, the default, the entire trajectory will be loaded.
        If supplied, ``stride`` will be ignored.

    Examples
    --------
    >>> import mdtraj as md
    >>> traj = md.load_hdf5('output.h5')
    >>> print traj
    <mdtraj.Trajectory with 500 frames, 423 atoms at 0x110740a90>

    >>> traj2 = md.load_hdf5('output.h5', stride=2, top='topology.pdb')
    >>> print traj2
    <mdtraj.Trajectory with 250 frames, 423 atoms at 0x11136e410>

    Returns
    -------
    trajectory : md.Trajectory
        The resulting trajectory, as an md.Trajectory object.

    See Also
    --------
    mdtraj.HDF5TrajectoryFile :  Low level interface to HDF5 files
    """
    if not isinstance(filename, (string_types, os.PathLike)):
        raise TypeError('filename must be of type path-like for load_lh5. '
            'you supplied %s' % type(filename))

    atom_indices = cast_indices(atom_indices)

    with HDF5TrajectoryFile(filename) as f:
        if frame is not None:
            f.seek(frame)
            n_frames = 1
        else:
            n_frames = None
        return f.read_as_traj(n_frames=n_frames, stride=stride, atom_indices=atom_indices)


@FormatRegistry.register_fileobject('.h5')
@FormatRegistry.register_fileobject('.hdf5')
class HDF5TrajectoryFile(object):
    """Interface for reading and writing to a MDTraj HDF5 molecular
    dynamics trajectory file, whose format is described
    `here <https://github.com/rmcgibbo/mdtraj/issues/36>`_.

    This is a file-like object, that both reading or writing depending
    on the `mode` flag. It implements the context manager protocol,
    so you can also use it with the python 'with' statement.

    The format is extremely flexible and high performance. It can hold a wide
    variety of information about a trajectory, including fields like the
    temperature and energies. Because it's built on the fantastic HDF5 library,
    it's easily extensible too.

    Parameters
    ----------
    filename : path-like
        Path to the file to open
    mode :  {'r, 'w'}
        Mode in which to open the file. 'r' is for reading and 'w' is for
        writing
    force_overwrite : bool
        In mode='w', how do you want to behave if a file by the name of `filename`
        already exists? if `force_overwrite=True`, it will be overwritten.
    compression : {'zlib', None}
        Apply compression to the file? This will save space, and does not
        cost too many cpu cycles, so it's recommended.

    Attributes
    ----------
    root
    title
    application
    topology
    randomState
    forcefield
    reference
    constraints

    See Also
    --------
    mdtraj.load_hdf5 : High-level wrapper that returns a ``md.Trajectory``
    """
    distance_unit = 'nanometers'

    def __init__(self, filename, mode='r', force_overwrite=True, compression='zlib'):
        self._open = False  # is the file handle currently open?
        self.mode = mode  # the mode in which the file was opened?

        if not mode in ['r', 'w', 'a']:
            raise ValueError("mode must be one of ['r', 'w', 'a']")

        if mode == 'w' and not force_overwrite and os.path.exists(filename):
            raise IOError('"%s" already exists' % filename)

        # import tables
        self.tables = import_('tables')

        if compression == 'zlib':
            compression = self.tables.Filters(complib='zlib', shuffle=True, complevel=1)
        elif compression is None:
            compression = None
        else:
            raise ValueError('compression must be either "zlib" or None')

        self._handle = self._open_file(filename, mode=mode, filters=compression)
        self._open = True

        if mode == 'w':
            # what frame are we currently reading or writing at?
            self._frame_index = 0
            # do we need to write the header information?
            self._needs_initialization = True
            if not os.fspath(filename).endswith('.h5'):
                warnings.warn('The .h5 extension is recommended.')

        elif mode == 'a':
            try:
                self._frame_index = len(self._handle.root.coordinates)
                self._needs_initialization = False
            except self.tables.NoSuchNodeError:
                self._frame_index = 0
                self._needs_initialization = True
        elif mode == 'r':
            self._frame_index = 0
            self._needs_initialization = False

    @property
    def root(self):
        """Direct access to the root group of the underlying Tables HDF5 file handle.

        This can be used for random or specific access to the underlying arrays
        on disk
        """
        _check_mode(self.mode, ('r', 'a'))
        return self._handle.root

    #####################################################
    # title global attribute (optional)
    #####################################################

    @property
    def title(self):
        """User-defined title for the data represented in the file"""
        if hasattr(self._handle.root._v_attrs, 'title'):
            return str(self._handle.root._v_attrs.title)
        return None

    @title.setter
    def title(self, value):
        """Set the user-defined title for the data represented in the file"""
        _check_mode(self.mode, ('w', 'a'))
        self._handle.root._v_attrs.title = str(value)

    #####################################################
    # application global attribute (optional)
    #####################################################

    @property
    def application(self):
        "Suite of programs that created the file"
        if hasattr(self._handle.root._v_attrs, 'application'):
            return str(self._handle.root._v_attrs.application)
        return None

    @application.setter
    def application(self, value):
        "Set the suite of programs that created the file"
        _check_mode(self.mode, ('w', 'a'))
        self._handle.root._v_attrs.application = str(value)

    #####################################################
    # topology global attribute (optional, recommended)
    #####################################################

    @property
    def topology(self):
        """Get the topology out from the file

        Returns
        -------
        topology : mdtraj.Topology
            A topology object
        """
        try:
            raw = self._get_node('/', name='topology')[0]
            if not isinstance(raw, string_types):
                raw = raw.decode()
            topology_dict = json.loads(raw)
        except self.tables.NoSuchNodeError:
            return None

        topology = Topology()

        for chain_dict in sorted(topology_dict['chains'], key=operator.itemgetter('index')):
            chain = topology.add_chain()
            for residue_dict in sorted(chain_dict['residues'], key=operator.itemgetter('index')):
                try:
                    resSeq = residue_dict["resSeq"]
                except KeyError:
                    resSeq = None
                    warnings.warn('No resSeq information found in HDF file, defaulting to zero-based indices')
                try:
                    segment_id = residue_dict["segmentID"]
                except KeyError:
                    segment_id = ""
                residue = topology.add_residue(residue_dict['name'], chain, resSeq=resSeq, segment_id=segment_id)
                for atom_dict in sorted(residue_dict['atoms'], key=operator.itemgetter('index')):
                    try:
                        element = elem.get_by_symbol(atom_dict['element'])
                    except KeyError:
                        element = elem.virtual
                    topology.add_atom(atom_dict['name'], element, residue)

        atoms = list(topology.atoms)
        for index1, index2 in topology_dict['bonds']:
            topology.add_bond(atoms[index1], atoms[index2])

        return topology

    @topology.setter
    def topology(self, topology_object):
        """Set the topology in the file

        Parameters
        ----------
        topology_object : mdtraj.Topology
            A topology object
        """
        _check_mode(self.mode, ('w', 'a'))

        # we want to be able to handle the simtk.openmm Topology object
        # here too, so if it's not an mdtraj topology we'll just guess
        # that it's probably an openmm topology and convert
        if not isinstance(topology_object, Topology):
            topology_object = Topology.from_openmm(topology_object)

        try:
            topology_dict = {
                'chains': [],
                'bonds': []
            }

            for chain in topology_object.chains:
                chain_dict = {
                    'residues': [],
                    'index': int(chain.index)
                }
                for residue in chain.residues:
                    residue_dict = {
                        'index': int(residue.index),
                        'name': str(residue.name),
                        'atoms': [],
                        "resSeq": int(residue.resSeq),
                        "segmentID": str(residue.segment_id)
                    }

                    for atom in residue.atoms:

                        try:
                            element_symbol_string = str(atom.element.symbol)
                        except AttributeError:
                            element_symbol_string = ""

                        residue_dict['atoms'].append({
                            'index': int(atom.index),
                            'name': str(atom.name),
                            'element': element_symbol_string
                        })
                    chain_dict['residues'].append(residue_dict)
                topology_dict['chains'].append(chain_dict)

            for atom1, atom2 in topology_object.bonds:
                topology_dict['bonds'].append([
                    int(atom1.index),
                    int(atom2.index)
                ])

        except AttributeError as e:
            raise AttributeError('topology_object fails to implement the'
                'chains() -> residue() -> atoms() and bond() protocol. '
                'Specifically, we encountered the following %s' % e)

        # actually set the tables
        try:
            self._remove_node(where='/', name='topology')
        except self.tables.NoSuchNodeError:
            pass

        data = json.dumps(topology_dict)
        if not isinstance(data, bytes):
            data = data.encode('ascii')

        if self.tables.__version__ >= '3.0.0':
            self._handle.create_array(where='/', name='topology', obj=[data])
        else:
            self._handle.createArray(where='/', name='topology', object=[data])

    #####################################################
    # randomState global attribute (optional)
    #####################################################

    @property
    def randomState(self):
        "State of the creators internal random number generator at the start of the simulation"
        if hasattr(self._handle.root._v_attrs, 'randomState'):
            return str(self._handle.root._v_attrs.randomState)
        return None

    @randomState.setter
    def randomState(self, value):
        "Set the state of the creators internal random number generator at the start of the simulation"
        _check_mode(self.mode, ('w', 'a'))
        self._handle.root._v_attrs.randomState = str(value)

    #####################################################
    # forcefield global attribute (optional)
    #####################################################

    @property
    def forcefield(self):
        "Description of the hamiltonian used. A short, human readable string, like AMBER99sbildn."
        if hasattr(self._handle.root._v_attrs, 'forcefield'):
            return str(self._handle.root._v_attrs.forcefield)
        return None

    @forcefield.setter
    def forcefield(self, value):
        "Set the description of the hamiltonian used. A short, human readable string, like AMBER99sbildn."
        _check_mode(self.mode, ('w', 'a'))
        self._handle.root._v_attrs.forcefield = str(value)

    #####################################################
    # reference global attribute (optional)
    #####################################################

    @property
    def reference(self):
        "A published reference that documents the program or parameters used to generate the data"
        if hasattr(self._handle.root._v_attrs, 'reference'):
            return str(self._handle.root._v_attrs.reference)
        return None

    @reference.setter
    def reference(self, value):
        "Set a published reference that documents the program or parameters used to generate the data"
        _check_mode(self.mode, ('w', 'a'))
        self._handle.root._v_attrs.reference = str(value)

    #####################################################
    # constraints array (optional)
    #####################################################

    @property
    def constraints(self):
        """Constraints applied to the bond lengths

        Returns
        -------
        constraints : {None, np.array, dtype=[('atom1', '<i4'), ('atom2', '<i4'), ('distance', '<f4')])}
            A one dimensional array with the a int, int, float type giving
            the index of the two atoms involved in the constraints and the
            distance of the constraint. If no constraint information
            is in the file, the return value is None.
        """
        if hasattr(self._handle.root, 'constraints'):
            return self._handle.root.constraints[:]
        return None

    @constraints.setter
    def constraints(self, value):
        """Set the constraints applied to bond lengths

        Returns
        -------
        valu : np.array, dtype=[('atom1', '<i4'), ('atom2', '<i4'), ('distance', '<f4')])
            A one dimensional array with the a int, int, float type giving
            the index of the two atoms involved in the constraints and the
            distance of the constraint.
        """
        _check_mode(self.mode, ('w', 'a'))

        dtype = np.dtype([
                ('atom1', np.int32),
                ('atom2', np.int32),
                ('distance', np.float32)
        ])
        if not value.dtype == dtype:
            raise ValueError('Constraints must be an array with dtype=%s. '
                             'currently, I don\'t do any casting' % dtype)

        if not hasattr(self._handle.root, 'constraints'):
            self._create_table(where='/', name='constraints',
                               description=dtype)

        self._handle.root.constraints.truncate(0)
        self._handle.root.constraints.append(value)

    #####################################################
    # read/write methods for file-like behavior
    #####################################################

    def read_as_traj(self, n_frames=None, stride=None, atom_indices=None):
        """Read a trajectory from the HDF5 file

        Parameters
        ----------
        n_frames : {int, None}
            The number of frames to read. If not supplied, all of the
            remaining frames will be read.
        stride : {int, None}
            By default all of the frames will be read, but you can pass this
            flag to read a subset of of the data by grabbing only every
            `stride`-th frame from disk.
        atom_indices : {int, None}
            By default all of the atom  will be read, but you can pass this
            flag to read only a subsets of the atoms for the `coordinates` and
            `velocities` fields. Note that you will have to carefully manage
            the indices and the offsets, since the `i`-th atom in the topology
            will not necessarily correspond to the `i`-th atom in your subset.

        Returns
        -------
        trajectory : Trajectory
            A trajectory object containing the loaded portion of the file.
        """
        _check_mode(self.mode, ('r',))

        from mdtraj.core.trajectory import Trajectory
        topology = self.topology
        if atom_indices is not None:
            topology = topology.subset(atom_indices)

        initial = int(self._frame_index)
        data = self.read(n_frames=n_frames, stride=stride, atom_indices=atom_indices)
        if len(data) == 0:
            return Trajectory(xyz=np.zeros((0, topology.n_atoms, 3)), topology=topology)

        in_units_of(data.coordinates, self.distance_unit, Trajectory._distance_unit, inplace=True)
        in_units_of(data.cell_lengths, self.distance_unit, Trajectory._distance_unit, inplace=True)

        return Trajectory(xyz=data.coordinates, topology=topology, time=data.time,
                          unitcell_lengths=data.cell_lengths,
                          unitcell_angles=data.cell_angles)

    def read(self, n_frames=None, stride=None, atom_indices=None):
        """Read one or more frames of data from the file

        Parameters
        ----------
        n_frames : {int, None}
            The number of frames to read. If not supplied, all of the
            remaining frames will be read.
        stride : {int, None}
            By default all of the frames will be read, but you can pass this
            flag to read a subset of of the data by grabbing only every
            `stride`-th frame from disk.
        atom_indices : {int, None}
            By default all of the atom  will be read, but you can pass this
            flag to read only a subsets of the atoms for the `coordinates` and
            `velocities` fields. Note that you will have to carefully manage
            the indices and the offsets, since the `i`-th atom in the topology
            will not necessarily correspond to the `i`-th atom in your subset.

        Notes
        -----
        If you'd like more flexible access to the data, that is available by
        using the pytables group directly, which is accessible via the
        `root` property on this class.

        Returns
        -------
        frames : namedtuple
            The returned namedtuple will have the fields "coordinates", "time", "cell_lengths",
            "cell_angles", "velocities", "kineticEnergy", "potentialEnergy",
            "temperature" and "alchemicalLambda". Each of the fields in the
            returned namedtuple will either be a numpy array or None, dependening
            on if that data was saved in the trajectory. All of the data shall be
            n units of "nanometers", "picoseconds", "kelvin", "degrees" and
            "kilojoules_per_mole".
        """
        _check_mode(self.mode, ('r',))

        if n_frames is None:
            n_frames = np.inf
        if stride is not None:
            stride = int(stride)

        total_n_frames = len(self._handle.root.coordinates)
        frame_slice = slice(self._frame_index, min(self._frame_index + n_frames, total_n_frames), stride)
        if frame_slice.stop - frame_slice.start == 0:
            return []

        if atom_indices is None:
            # get all of the atoms
            atom_slice = slice(None)
        else:
            atom_slice = ensure_type(atom_indices, dtype=int, ndim=1,
                                     name='atom_indices', warn_on_cast=False)
            if not np.all(atom_slice < self._handle.root.coordinates.shape[1]):
                raise ValueError('As a zero-based index, the entries in '
                    'atom_indices must all be less than the number of atoms '
                    'in the trajectory, %d' % self._handle.root.coordinates.shape[1])
            if not np.all(atom_slice >= 0):
                raise ValueError('The entries in atom_indices must be greater '
                    'than or equal to zero')

        def get_field(name, slice, out_units, can_be_none=True):
            try:
                node = self._get_node(where='/', name=name)
                data = node.__getitem__(slice)
                in_units = node.attrs.units
                if not isinstance(in_units, string_types):
                    in_units = in_units.decode()
                data =  in_units_of(data, in_units, out_units)
                return data
            except self.tables.NoSuchNodeError:
                if can_be_none:
                    return None
                raise

        frames = Frames(
            coordinates = get_field('coordinates', (frame_slice, atom_slice, slice(None)),
                                    out_units='nanometers', can_be_none=False),
            time = get_field('time', frame_slice, out_units='picoseconds'),
            cell_lengths = get_field('cell_lengths', (frame_slice, slice(None)), out_units='nanometers'),
            cell_angles = get_field('cell_angles', (frame_slice, slice(None)), out_units='degrees'),
            velocities = get_field('velocities', (frame_slice, atom_slice, slice(None)), out_units='nanometers/picosecond'),
            kineticEnergy = get_field('kineticEnergy', frame_slice, out_units='kilojoules_per_mole'),
            potentialEnergy = get_field('potentialEnergy', frame_slice, out_units='kilojoules_per_mole'),
            temperature = get_field('temperature', frame_slice, out_units='kelvin'),
            alchemicalLambda = get_field('lambda', frame_slice, out_units='dimensionless')
        )

        self._frame_index += (frame_slice.stop - frame_slice.start)
        return frames

    def write(self, coordinates, time=None, cell_lengths=None, cell_angles=None,
                    velocities=None, kineticEnergy=None, potentialEnergy=None,
                    temperature=None, alchemicalLambda=None):
        """Write one or more frames of data to the file

        This method saves data that is associated with one or more simulation
        frames. Note that all of the arguments can either be raw numpy arrays
        or unitted arrays (with simtk.unit.Quantity). If the arrays are unittted,
        a unit conversion will be automatically done from the supplied units
        into the proper units for saving on disk. You won't have to worry about
        it.

        Furthermore, if you wish to save a single frame of simulation data, you
        can do so naturally, for instance by supplying a 2d array for the
        coordinates and a single float for the time. This "shape deficiency"
        will be recognized, and handled appropriately.

        Parameters
        ----------
        coordinates : np.ndarray, shape=(n_frames, n_atoms, 3)
            The cartesian coordinates of the atoms to write. By convention, the
            lengths should be in units of nanometers.
        time : np.ndarray, shape=(n_frames,), optional
            You may optionally specify the simulation time, in picoseconds
            corresponding to each frame.
        cell_lengths : np.ndarray, shape=(n_frames, 3), dtype=float32, optional
            You may optionally specify the unitcell lengths.
            The length of the periodic box in each frame, in each direction,
            `a`, `b`, `c`. By convention the lengths should be in units
            of angstroms.
        cell_angles : np.ndarray, shape=(n_frames, 3), dtype=float32, optional
            You may optionally specify the unitcell angles in each frame.
            Organized analogously to cell_lengths. Gives the alpha, beta and
            gamma angles respectively. By convention, the angles should be
            in units of degrees.
        velocities :  np.ndarray, shape=(n_frames, n_atoms, 3), optional
            You may optionally specify the cartesian components of the velocity
            for each atom in each frame. By convention, the velocities
            should be in units of nanometers / picosecond.
        kineticEnergy : np.ndarray, shape=(n_frames,), optional
            You may optionally specify the kinetic energy in each frame. By
            convention the kinetic energies should b in units of kilojoules per
            mole.
        potentialEnergy : np.ndarray, shape=(n_frames,), optional
            You may optionally specify the potential energy in each frame. By
            convention the kinetic energies should b in units of kilojoules per
            mole.
        temperature : np.ndarray, shape=(n_frames,), optional
            You may optionally specify the temperature in each frame. By
            convention the temperatures should b in units of Kelvin.
        alchemicalLambda : np.ndarray, shape=(n_frames,), optional
            You may optionally specify the alchemical lambda in each frame. These
            have no units, but are generally between zero and one.
        """
        _check_mode(self.mode, ('w', 'a'))

        # these must be either both present or both absent. since
        # we're going to throw an error if one is present w/o the other,
        # lets do it now.
        if cell_lengths is None and cell_angles is not None:
            raise ValueError('cell_lengths were given, but no cell_angles')
        if cell_lengths is not None and cell_angles is None:
            raise ValueError('cell_angles were given, but no cell_lengths')

        # if the input arrays are simtk.unit.Quantities, convert them
        # into md units. Note that this acts as a no-op if the user doesn't
        # have simtk.unit installed (e.g. they didn't install OpenMM)
        coordinates = in_units_of(coordinates, None, 'nanometers')
        time = in_units_of(time, None, 'picoseconds')
        cell_lengths = in_units_of(cell_lengths, None, 'nanometers')
        cell_angles = in_units_of(cell_angles, None, 'degrees')
        velocities = in_units_of(velocities, None, 'nanometers/picosecond')
        kineticEnergy = in_units_of(kineticEnergy, None, 'kilojoules_per_mole')
        potentialEnergy = in_units_of(potentialEnergy, None, 'kilojoules_per_mole')
        temperature = in_units_of(temperature, None, 'kelvin')
        alchemicalLambda = in_units_of(alchemicalLambda, None, 'dimensionless')

        # do typechecking and shapechecking on the arrays
        # this ensure_type method has a lot of options, but basically it lets
        # us validate most aspects of the array. Also, we can upconvert
        # on defficent ndim, which means that if the user sends in a single
        # frame of data (i.e. coordinates is shape=(n_atoms, 3)), we can
        # realize that. obviously the default mode is that they want to
        # write multiple frames at a time, so the coordinate shape is
        # (n_frames, n_atoms, 3)
        coordinates = ensure_type(coordinates, dtype=np.float32, ndim=3,
            name='coordinates', shape=(None, None, 3), can_be_none=False,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)
        n_frames, n_atoms, = coordinates.shape[0:2]
        time = ensure_type(time, dtype=np.float32, ndim=1,
            name='time', shape=(n_frames,), can_be_none=True,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)
        cell_lengths = ensure_type(cell_lengths, dtype=np.float32, ndim=2,
            name='cell_lengths', shape=(n_frames, 3), can_be_none=True,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)
        cell_angles = ensure_type(cell_angles, dtype=np.float32, ndim=2,
            name='cell_angles', shape=(n_frames, 3), can_be_none=True,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)
        velocities = ensure_type(velocities, dtype=np.float32, ndim=3,
            name='velocities', shape=(n_frames, n_atoms, 3), can_be_none=True,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)
        kineticEnergy = ensure_type(kineticEnergy, dtype=np.float32, ndim=1,
            name='kineticEnergy', shape=(n_frames,), can_be_none=True,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)
        potentialEnergy = ensure_type(potentialEnergy, dtype=np.float32, ndim=1,
            name='potentialEnergy', shape=(n_frames,), can_be_none=True,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)
        temperature = ensure_type(temperature, dtype=np.float32, ndim=1,
            name='temperature', shape=(n_frames,), can_be_none=True,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)
        alchemicalLambda = ensure_type(alchemicalLambda, dtype=np.float32, ndim=1,
            name='alchemicalLambda', shape=(n_frames,), can_be_none=True,
            warn_on_cast=False, add_newaxis_on_deficient_ndim=True)

        # if this is our first call to write(), we need to create the headers
        # and the arrays in the underlying HDF5 file
        if self._needs_initialization:
            self._initialize_headers(
                n_atoms=n_atoms,
                set_coordinates=True,
                set_time=(time is not None),
                set_cell=(cell_lengths is not None or cell_angles is not None),
                set_velocities=(velocities is not None),
                set_kineticEnergy=(kineticEnergy is not None),
                set_potentialEnergy=(potentialEnergy is not None),
                set_temperature=(temperature is not None),
                set_alchemicalLambda=(alchemicalLambda is not None))
            self._needs_initialization = False

            # we need to check that that the entries that the user is trying
            # to save are actually fields in OUR file

        try:
            # try to get the nodes for all of the fields that we have
            # which are not None
            for name in ['coordinates', 'time', 'cell_angles', 'cell_lengths',
                         'velocities', 'kineticEnergy', 'potentialEnergy', 'temperature']:
                contents = locals()[name]
                if contents is not None:
                    self._get_node(where='/', name=name).append(contents)
                if contents is None:
                    # for each attribute that they're not saving, we want
                    # to make sure the file doesn't explect it
                    try:
                        self._get_node(where='/', name=name)
                        raise AssertionError()
                    except self.tables.NoSuchNodeError:
                        pass


            # lambda is different, since the name in the file is lambda
            # but the name in this python function is alchemicalLambda
            name = 'lambda'
            if alchemicalLambda is not None:
                self._get_node(where='/', name=name).append(alchemicalLambda)
            else:
                try:
                    self._get_node(where='/', name=name)
                    raise AssertionError()
                except self.tables.NoSuchNodeError:
                    pass

        except self.tables.NoSuchNodeError:
            raise ValueError("The file that you're trying to save to doesn't "
                "contain the field %s. You can always save a new trajectory "
                "and have it contain this information, but I don't allow 'ragged' "
                "arrays. If one frame is going to have %s information, then I expect "
                "all of them to. So I can't save it for just these frames. Sorry "
                "about that :)" % (name, name))
        except AssertionError:
            raise ValueError("The file that you're saving to expects each frame "
                            "to contain %s information, but you did not supply it."
                            "I don't allow 'ragged' arrays. If one frame is going "
                            "to have %s information, then I expect all of them to. "
                            % (name, name))

        self._frame_index += n_frames
        self.flush()

    def _initialize_headers(self, n_atoms, set_coordinates, set_time, set_cell,
                            set_velocities, set_kineticEnergy, set_potentialEnergy,
                            set_temperature, set_alchemicalLambda):
        self._n_atoms = n_atoms

        self._handle.root._v_attrs.conventions = 'Pande'
        self._handle.root._v_attrs.conventionVersion = '1.1'
        self._handle.root._v_attrs.program = 'MDTraj'
        self._handle.root._v_attrs.programVersion = version.short_version
        self._handle.root._v_attrs.title = 'title'

        # if the client has not the title attribute themselves, we'll
        # set it to MDTraj as a default option.
        if not hasattr(self._handle.root._v_attrs, 'application'):
            self._handle.root._v_attrs.application = 'MDTraj'

        # create arrays that store frame level informat
        if set_coordinates:
            self._create_earray(where='/', name='coordinates',
                atom=self.tables.Float32Atom(), shape=(0, self._n_atoms, 3))
            self._handle.root.coordinates.attrs['units'] = 'nanometers'

        if set_time:
            self._create_earray(where='/', name='time',
                atom=self.tables.Float32Atom(), shape=(0,))
            self._handle.root.time.attrs['units'] = 'picoseconds'

        if set_cell:
            self._create_earray(where='/', name='cell_lengths',
                atom=self.tables.Float32Atom(), shape=(0, 3))
            self._create_earray(where='/', name='cell_angles',
                atom=self.tables.Float32Atom(), shape=(0, 3))
            self._handle.root.cell_lengths.attrs['units'] = 'nanometers'
            self._handle.root.cell_angles.attrs['units'] = 'degrees'

        if set_velocities:
            self._create_earray(where='/', name='velocities',
                atom=self.tables.Float32Atom(), shape=(0, self._n_atoms, 3))
            self._handle.root.velocities.attrs['units'] = 'nanometers/picosecond'

        if set_kineticEnergy:
            self._create_earray(where='/', name='kineticEnergy',
                atom=self.tables.Float32Atom(), shape=(0,))
            self._handle.root.kineticEnergy.attrs['units'] = 'kilojoules_per_mole'

        if set_potentialEnergy:
            self._create_earray(where='/', name='potentialEnergy',
                atom=self.tables.Float32Atom(), shape=(0,))
            self._handle.root.potentialEnergy.attrs['units'] = 'kilojoules_per_mole'

        if set_temperature:
            self._create_earray(where='/', name='temperature',
                atom=self.tables.Float32Atom(), shape=(0,))
            self._handle.root.temperature.attrs['units'] = 'kelvin'

        if set_alchemicalLambda:
            self._create_earray(where='/', name='lambda',
                atom=self.tables.Float32Atom(), shape=(0,))
            self._get_node('/', name='lambda').attrs['units'] = 'dimensionless'

    def seek(self, offset, whence=0):
        """Move to a new file position

        Parameters
        ----------
        offset : int
            A number of frames.
        whence : {0, 1, 2}
            0: offset from start of file, offset should be >=0.
            1: move relative to the current position, positive or negative
            2: move relative to the end of file, offset should be <= 0.
            Seeking beyond the end of a file is not supported
        """
        _check_mode(self.mode, ('r',))

        if whence == 0 and offset >= 0:
            self._frame_index = offset
        elif whence == 1:
            self._frame_index = self._frame_index + offset
        elif whence == 2 and offset <= 0:
            self._frame_index = len(self._handle.root.coordinates) + offset
        else:
            raise IOError('Invalid argument')

    def tell(self):
        """Current file position

        Returns
        -------
        offset : int
            The current frame in the file.
        """
        return int(self._frame_index)

    def _validate(self):
        raise NotImplementedError()

        # check that all of the shapes are consistent
        # check that everything has units

    @property
    def _get_node(self):
        if self.tables.__version__ >= '3.0.0':
            return self._handle.get_node
        return self._handle.getNode

    @property
    def _create_earray(self):
        if self.tables.__version__ >= '3.0.0':
            return self._handle.create_earray
        return self._handle.createEArray

    @property
    def _create_table(self):
        if self.tables.__version__ >= '3.0.0':
            return self._handle.create_table
        return self._handle.createTable

    @property
    def _remove_node(self):
        if self.tables.__version__ >= '3.0.0':
            return self._handle.remove_node
        return self._handle.removeNode

    @property
    def _open_file(self):
        if self.tables.__version__ >= '3.0.0':
           return self.tables.open_file
        return self.tables.openFile

    def close(self):
        "Close the HDF5 file handle"
        if self._open:
            self._handle.close()
            self._open = False

    def flush(self):
        "Write all buffered data in the to the disk file."
        if self._open:
            self._handle.flush()

    def __del__(self):
        self.close()

    def __enter__(self):
        "Support the context manager protocol"
        return self

    def __exit__(self, *exc_info):
        "Support the context manager protocol"
        self.close()

    def __len__(self):
        "Number of frames in the file"
        if not self._open:
            raise ValueError('I/O operation on closed file')
        return len(self._handle.root.coordinates)


def _check_mode(m, modes):
    if m not in modes:
        raise ValueError('This operation is only available when a file '
                         'is open in mode="%s".' % m)



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2013 Stanford University and the Authors
#
# Authors: Alexander Yang
# Contributors:
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################


import numpy as np
import mdtraj
from mdtraj import io
from mdtraj.testing import eq
import pytest

def test_read(get_fn):
    filename = get_fn('out.gsd')
    traj = mdtraj.load(filename)

    assert len(traj) == 1000
    assert traj.top.n_atoms == 80
    assert traj.top.n_bonds == 70
    assert traj.top.atom(0).name == 'opls_135'
    assert traj.top.atom(1).name == 'opls_140'

def test_read_start(get_fn):
    filename = get_fn('out.gsd')
    traj = mdtraj.load(filename)
    other = mdtraj.load(filename, start=2)

    eq(traj[2].xyz, other[0].xyz)
    eq(traj[2].unitcell_lengths, other[0].unitcell_lengths)
    assert traj.top == other.top

def test_read_frame(get_fn):
    filename = get_fn('out.gsd')
    traj = mdtraj.load(filename)
    other = mdtraj.load(filename, frame=2)

    eq(traj[2].xyz, other[0].xyz)
    eq(traj[2].unitcell_lengths, other[0].unitcell_lengths)
    assert traj.top == other.top

def test_read_stride(get_fn):
    filename = get_fn('out.gsd')
    traj = mdtraj.load(filename)
    other = mdtraj.load(filename, stride=10)

    eq(traj[0].xyz, other[0].xyz)
    eq(traj[0].unitcell_lengths, other[0].unitcell_lengths)

    eq(traj[10].xyz, other[1].xyz)
    eq(traj[10].unitcell_lengths, other[1].unitcell_lengths)

    assert traj.top == other.top

def test_read_variable_top_error(get_fn):
    filename = get_fn('variable_top.gsd')
    with pytest.raises(IOError):
        traj = mdtraj.load(filename)


def test_write(get_fn, tmpdir):
    filename = get_fn('out.gsd')
    traj = mdtraj.load(filename)
    fn = '{}/compare.gsd'.format(tmpdir)
    traj.save(fn)
    other = mdtraj.load(fn)

    assert traj.top == other.top
    eq(other.xyz, traj.xyz)
    eq(other.unitcell_lengths, traj.unitcell_lengths)

def test_write_frame(get_fn, tmpdir):
    filename = get_fn('out.gsd')
    traj = mdtraj.load(filename)
    fn = '{}/compare.gsd'.format(tmpdir)
    traj[2].save(fn)
    other = mdtraj.load(fn)

    assert traj.top == other.top
    eq(traj[2].xyz, other.xyz)
    eq(traj[2].unitcell_lengths, other.unitcell_lengths)




##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2017 Stanford University and the Authors
#
# Authors: Lee-Ping Wang
# Contributors: Robert McGibbon
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################


import mdtraj as md
import numpy as np
from mdtraj.formats import ArcTrajectoryFile
from mdtraj.formats import PDBTrajectoryFile
from mdtraj.testing import eq


def test_read_0(get_fn):
    with ArcTrajectoryFile(get_fn('4waters.arc')) as f:
        xyz, leng, ang = f.read()
    with PDBTrajectoryFile(get_fn('4waters.pdb')) as f:
        xyz2 = f.positions
    eq(xyz, xyz2, decimal=3)

def test_read_arctraj(get_fn):
    traj = md.load(get_fn('nitrogen.arc'), top=get_fn('nitrogen.pdb'))
    owntop = md.load(get_fn('nitrogen.arc'))
    eq(traj.xyz, owntop.xyz)

def test_read_pbc(get_fn):
    with ArcTrajectoryFile(get_fn('thf200.arc')) as f:
       xyz, leng, ang = f.read()
    eq(xyz[0, 0], np.array([21.231166, 32.549819, 8.278454]), decimal=3)
    eq(xyz[0,-1], np.array([29.013880, 6.255754, 44.074519]), decimal=3)
    eq(leng, np.array([[45.119376, 45.119376, 45.119376]]), decimal=3)
    eq(ang, np.array([[90.0, 90.0, 90.0]]), decimal=3)



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2013 Stanford University and the Authors
#
# Authors: Gregory Bowman
# Contributors: Robert McGibbon
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################

import os
import tempfile
import unittest

import numpy as np
import tables

from mdtraj import io
from mdtraj.testing import eq

fd, temp = tempfile.mkstemp(suffix='.h5')
os.close(fd)


def teardown_module(module):
    """remove the temporary file created by tests in this file
    this gets automatically called by pytest"""
    os.unlink(temp)


def test_overwrite_1():
    fid, fn = tempfile.mkstemp()
    try:
        a = np.arange(10)
        b = a + 1
        io.saveh(fn, a=a)
        io.saveh(fn, b=b)
        eq(io.loadh(fn, 'a'), a)
        eq(io.loadh(fn, 'b'), b)
    except:
        raise
    finally:
        if os.path.exists(fn):
            os.close(fid)
            os.unlink(fn)


def test_overwrite_2():
    fid, fn = tempfile.mkstemp()
    try:
        a = np.arange(10)
        b = a + 1
        io.saveh(fn, a=a)
        io.saveh(fn, a=b)
        eq(io.loadh(fn, 'a'), b)
    except:
        raise
    finally:
        if os.path.exists(fn):
            os.close(fid)
            os.unlink(fn)


class test_io(unittest.TestCase):
    def setUp(self):
        # setup() is called before very test and just creates a
        # temporary work space for reading/writing files.
        fid, self.filename1 = tempfile.mkstemp()
        fid, self.filename2 = tempfile.mkstemp()
        self.data = np.arange(10000, dtype=np.float32)

        # Write Data to an HDF5 file as a compressed CArray.
        hdf_file = tables.open_file(self.filename1, 'a')
        hdf_file.create_carray("/", "arr_0", tables.Float32Atom(),
                               self.data.shape, filters=io.COMPRESSION)
        hdf_file.root.arr_0[:] = self.data[:]
        hdf_file.flush()
        hdf_file.close()

    def test_load_1(self):
        # Load by specifying array name
        TestData = io.loadh(self.filename1, 'arr_0')
        eq(TestData, self.data)

    def test_load_2(self):
        # load using deferred=False
        TestData = io.loadh(self.filename1, deferred=False)['arr_0']
        eq(TestData, self.data)

    def test_load_3(self):
        # load using deferred=True
        deferred = io.loadh(self.filename1, deferred=True)
        eq(deferred['arr_0'], self.data)
        deferred.close()

    def test_save(self):
        # Save HDF5 to disk and load it back up
        io.saveh(self.filename2, self.data)
        TestData = io.loadh(self.filename2, 'arr_0')
        eq(TestData, self.data)

    def teardown(self):
        os.remove(self.filename1)
        os.remove(self.filename2)


class test_io_int(test_io):
    "Run the same test as the class above, but using int64 data"

    def setUp(self):
        # setup() is called before very test and just creates
        # a temporary work space for reading/writing files.
        fid, self.filename1 = tempfile.mkstemp()
        fid, self.filename2 = tempfile.mkstemp()
        self.data = np.arange(10000, dtype=np.int64)

        # Write Data to an HDF5 file as a compressed CArray.
        hdf_file = tables.open_file(self.filename1, 'a')
        hdf_file.create_carray("/", "arr_0", tables.Int64Atom(),
                               self.data.shape, filters=io.COMPRESSION)
        hdf_file.root.arr_0[:] = self.data[:]
        hdf_file.flush()
        hdf_file.close()


def test_groups():
    # Test to ensure that files are loaded correctly even if they contain
    # nested groups and stuff
    x = np.random.randn(10)
    y = np.random.randn(11)
    f = tables.open_file(temp, 'w')
    f.create_group(where='/', name='mygroup')
    f.create_array(where='/mygroup', name='myarray', obj=x)
    f.create_array(where='/', name='mya2', obj=y)
    f.close()

    assert eq(io.loadh(temp)['mygroup/myarray'], x)
    assert eq(io.loadh(temp)['mya2'], y)
    assert eq(io.loadh(temp, deferred=False)['mygroup/myarray'], x)
    assert eq(io.loadh(temp, deferred=False)['mya2'], y)
    assert eq(io.loadh(temp, 'mygroup/myarray'), x)
    assert eq(io.loadh(temp, 'mya2'), y)



cdef extern from "include/molfile_plugin.h":
    ctypedef struct molfile_timestep_t:
      float *coords  # coordinates of all atoms, arranged xyzxyzxyz
      float *velocities  # space for velocities of all atoms; same layout
      float A, B, C, alpha, beta, gamma
      double physical_time

    ctypedef size_t molfile_ssize_t

    ctypedef struct molfile_timestep_metadata:
      unsigned int count                  # total # timesteps; -1 if unknown
      unsigned int avg_bytes_per_timestep # bytes per timestep
      int has_velocities

cdef extern from "include/dtrplugin.hxx":

    void* open_file_read(const char *path, const char *filetype, int *natoms)
    int read_timestep2(void *v, molfile_ssize_t n, molfile_timestep_t *ts)
    molfile_ssize_t read_times(void *v, molfile_ssize_t start, molfile_ssize_t count, double *times)
    void close_file_read(void *v)

    void* open_file_write(const char *path, const char *filetype, const int natoms)
    int write_timestep(void *v, const molfile_timestep_t *ts)
    void close_file_write(void *v)
    int read_timestep_metadata(void *v, molfile_timestep_metadata *m)
    ssize_t dtr_curframe(void* v)




import numpy as np
import mdtraj as md
from mdtraj.testing import eq

random = np.random.RandomState(0)


def compute_neighbors_reference(traj, cutoff, query_indices, haystack_indices=None):
    if haystack_indices is None:
        haystack_indices = range(traj.n_atoms)
    # explicitly enumerate the pairs of query-haystack indices we need to
    # check
    pairs = np.array([(q, i) for i in haystack_indices for q in query_indices if i != q])
    dists = md.compute_distances(traj, pairs)
    # some of the haystack might be within cutoff of more than one of the
    # query atoms, so we need unique
    reference = [np.unique(pairs[dists[i] < cutoff, 1]) for i in range(traj.n_frames)]
    return reference


def test_compute_neighbors_1():
    n_frames = 2
    n_atoms = 20
    cutoff = 2
    xyz = random.randn(n_frames, n_atoms, 3)
    traj = md.Trajectory(xyz=xyz, topology=None)

    query_indices = [0, 1]
    value = md.compute_neighbors(traj, cutoff, query_indices)
    reference = compute_neighbors_reference(traj, cutoff, query_indices)

    for i in range(n_frames):
        eq(value[i], reference[i])


def test_compute_neighbors_2(get_fn):
    traj = md.load(get_fn('4ZUO.pdb'))
    query_indices = traj.top.select('residue 1')
    cutoff = 1.0
    value = md.compute_neighbors(traj, cutoff, query_indices)
    reference = compute_neighbors_reference(traj, cutoff, query_indices)
    for i in range(traj.n_frames):
        eq(value[i], reference[i])


def test_compute_neighbors_3(get_fn):
    traj = md.load(get_fn('test_good.nc'), top=get_fn('test.parm7'))
    query_indices = traj.top.select('residue 1')
    cutoff = 1.0
    value = md.compute_neighbors(traj, cutoff, query_indices)
    reference = compute_neighbors_reference(traj, cutoff, query_indices)
    for i in range(traj.n_frames):
        eq(value[i], reference[i])



##############################################################################
# MDTraj: A Python Library for Loading, Saving, and Manipulating
#         Molecular Dynamics Trajectories.
# Copyright 2012-2013 Stanford University and the Authors

# Authors: Christoph Klein
# Contributors:
#
# MDTraj is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
##############################################################################


import tempfile, os

import numpy as np

import mdtraj as md
from mdtraj.formats import LAMMPSTrajectoryFile
from mdtraj.testing import eq


fd, temp = tempfile.mkstemp(suffix='.lammpstrj')
def teardown_module(module):
    """Remove the temporary file created by tests in this file
    this gets automatically called by pytest. """
    os.close(fd)
    os.unlink(temp)

def test_read_0(get_fn):
    with LAMMPSTrajectoryFile(get_fn('frame0.lammpstrj')) as f:
        xyz, _, _ = f.read()
    with LAMMPSTrajectoryFile(get_fn('frame0.lammpstrj')) as f:
        xyz3, _, _ = f.read(stride=3)
    eq(xyz[::3], xyz3)

def test_read_1(get_fn):
    reference = md.load(get_fn('frame0.dcd'), top=get_fn('native.pdb'))
    traj = md.load(get_fn('frame0.lammpstrj'), top=get_fn('native.pdb'))

    eq(reference.xyz[0], traj.xyz[0], decimal=3)

def test_read_write_0():
    xyz = 10 * np.random.randn(100, 11, 3)
    lengths = np.ones(shape=(100, 3))
    angles = np.empty(shape=(100, 3))
    angles.fill(45)

    with LAMMPSTrajectoryFile(temp, mode='w') as f:
        f.write(xyz, lengths, angles)
    with LAMMPSTrajectoryFile(temp) as f:
        xyz2, new_lengths, new_angles = f.read()

    eq(lengths, new_lengths)
    eq(angles, new_angles)
    eq(xyz, xyz2, decimal=3)

def test_mdwrite(get_fn):
    t = md.load(get_fn('frame0.dcd'), top=get_fn('native.pdb'))
    t.save(temp)

def test_multiread(get_fn):
    reference = md.load(get_fn('frame0.lammpstrj'), top=get_fn('native.pdb'))
    with LAMMPSTrajectoryFile(get_fn('frame0.lammpstrj')) as f:
        xyz0, _, _ = f.read(n_frames=1)
        xyz1, _, _ = f.read(n_frames=1)

    eq(reference.xyz[0], xyz0[0]/10)
    eq(reference.xyz[1], xyz1[0]/10)

def test_seek(get_fn):
    reference = md.load(get_fn('frame0.lammpstrj'), top=get_fn('native.pdb'))

    with LAMMPSTrajectoryFile(get_fn('frame0.lammpstrj')) as f:
        f.seek(1)
        eq(1, f.tell())
        xyz1, _, _ = f.read(n_frames=1)
        eq(reference.xyz[1], xyz1[0]/10)

        f.seek(10)
        eq(10, f.tell())
        xyz10, _, _ = f.read(n_frames=1)
        eq(reference.xyz[10], xyz10[0]/10)
        eq(11, f.tell())

        f.seek(-8, 1)
        xyz3, _, _ = f.read(n_frames=1)
        eq(reference.xyz[3], xyz3[0]/10)

        f.seek(4, 1)
        xyz8, _, _ = f.read(n_frames=1)
        eq(reference.xyz[8], xyz8[0]/10)

def test_custom(get_fn):
    t0 = md.load(get_fn('custom.lammpstrj'), top=get_fn('custom.pdb'))
    t0.save(temp)

    t1 = md.load(temp, top=get_fn('custom.pdb'))
    eq(t0.xyz, t1.xyz)



import numpy as np
from mdtraj.testing import eq
from mdtraj.utils import uniform_quaternion, rotation_matrix_from_quaternion

def test_quaternion_0():
    q = uniform_quaternion(random_state=2)
    eq(np.sum(q**2), np.float64(1.0))
    eq(q.shape, (4,))

    q2 = uniform_quaternion(size=2)
    eq(np.sum(q2**2, axis=1), np.ones(2))
    eq(q2.shape, (2,4))

    q2 = uniform_quaternion(size=(6,6))
    eq(np.sum(q2**2, axis=2), np.ones((6,6)))
    eq(q2.shape, (6,6,4))


def test_quaternion_1():
    q = uniform_quaternion(random_state=2)
    rot = rotation_matrix_from_quaternion(q)
    
    eq(rot.shape, (3,3))
    eq(np.linalg.det(rot), np.double(1.0))
    eq(np.linalg.inv(rot), rot.T)


def test_quaternion_2():
    q = uniform_quaternion(size=2)
    rot = rotation_matrix_from_quaternion(q)
    
    eq(rot.shape, (2, 3, 3))

    # check 1st rotation matrix
    eq(np.linalg.det(rot[0]), np.double(1.0))
    eq(np.linalg.inv(rot[0]), rot[0].T)
    # check 2nd rotation matrix
    eq(np.linalg.det(rot[1]), np.double(1.0))
    eq(np.linalg.inv(rot[1]), rot[1].T)
